class Emojis {
  // Let's create a singleton
  static final Emojis _emojis = Emojis._internal();
  factory Emojis() {
    return _emojis;
  }
  Map<String, String> _reversedEmojiMap;
  Emojis._internal() {
    // reverse the map for reverse lookups
    _reversedEmojiMap = _EMOJIS.map((k, v) => MapEntry(v, k));
  }

  String getByName(String name) {
    // Stupid case when emoji names aren't names but emojis themselves
    name = name.replaceAll(':', '');
    List<int> codePoints = [];
    String codePoint = _EMOJIS[name];
    if (codePoint == null) {
      if (reverseLookup(name) != null)
        return name;
      else {
        codePoint = _EMOJIS['question'];
      }
    }
    for (String cp in codePoint.split('-')) {
      codePoints.add(int.parse(cp, radix: 16));
    }
    return String.fromCharCodes(codePoints);
  }

  String reverseLookup(String value) {
    String hexcode =
        value.runes.map((r) => r.toRadixString(16).toUpperCase()).join('-');
    // print('hexcode is: $hexcode');
    return _reversedEmojiMap[hexcode];
  }

  // String getClosestMatch(String name) {
  //   name = name.replaceAll(':', '');
  //   if (_reversedEmojiMap[name] != null) return name;
  //   var res = _EMOJIS[name];
  //
  //   // That's a pretty dumb algorithm, but will do for now
  //   // TODO search a better solution to get closest match emoji
  //   if (res == null) {
  //     final parts = name.split('_');
  //     for (var i = 0; i < parts.length; i++) {
  //       final emoji = _EMOJIS.keys
  //           .firstWhere((k) => k.contains(parts[i]), orElse: () => null);
  //       if (emoji != null) {
  //         res = _EMOJIS[emoji];
  //         break;
  //       }
  //     }
  //   }
  //   return res ?? _EMOJIS['question'];
  // }

  // https://raw.githubusercontent.com/omnidan/node-emoji/master/lib/emoji.json
  static const Map<String, String> _EMOJIS = {
    "100": "1F4AF",
    "1234": "1F522",
    "hash": "0023-FE0F-20E3",
    "keycap_star": "002A-FE0F-20E3",
    "zero": "0030-FE0F-20E3",
    "one": "0031-FE0F-20E3",
    "two": "0032-FE0F-20E3",
    "three": "0033-FE0F-20E3",
    "four": "0034-FE0F-20E3",
    "five": "0035-FE0F-20E3",
    "six": "0036-FE0F-20E3",
    "seven": "0037-FE0F-20E3",
    "eight": "0038-FE0F-20E3",
    "nine": "0039-FE0F-20E3",
    "copyright": "00A9-FE0F",
    "registered": "00AE-FE0F",
    "mahjong": "1F004",
    "black_joker": "1F0CF",
    "a": "1F170-FE0F",
    "b": "1F171-FE0F",
    "o2": "1F17E-FE0F",
    "parking": "1F17F-FE0F",
    "ab": "1F18E",
    "cl": "1F191",
    "cool": "1F192",
    "free": "1F193",
    "id": "1F194",
    "new": "1F195",
    "ng": "1F196",
    "ok": "1F197",
    "sos": "1F198",
    "up": "1F199",
    "vs": "1F19A",
    "flag-ac": "1F1E6-1F1E8",
    "flag-ad": "1F1E6-1F1E9",
    "flag-ae": "1F1E6-1F1EA",
    "flag-af": "1F1E6-1F1EB",
    "flag-ag": "1F1E6-1F1EC",
    "flag-ai": "1F1E6-1F1EE",
    "flag-al": "1F1E6-1F1F1",
    "flag-am": "1F1E6-1F1F2",
    "flag-ao": "1F1E6-1F1F4",
    "flag-aq": "1F1E6-1F1F6",
    "flag-ar": "1F1E6-1F1F7",
    "flag-as": "1F1E6-1F1F8",
    "flag-at": "1F1E6-1F1F9",
    "flag-au": "1F1E6-1F1FA",
    "flag-aw": "1F1E6-1F1FC",
    "flag-ax": "1F1E6-1F1FD",
    "flag-az": "1F1E6-1F1FF",
    "flag-ba": "1F1E7-1F1E6",
    "flag-bb": "1F1E7-1F1E7",
    "flag-bd": "1F1E7-1F1E9",
    "flag-be": "1F1E7-1F1EA",
    "flag-bf": "1F1E7-1F1EB",
    "flag-bg": "1F1E7-1F1EC",
    "flag-bh": "1F1E7-1F1ED",
    "flag-bi": "1F1E7-1F1EE",
    "flag-bj": "1F1E7-1F1EF",
    "flag-bl": "1F1E7-1F1F1",
    "flag-bm": "1F1E7-1F1F2",
    "flag-bn": "1F1E7-1F1F3",
    "flag-bo": "1F1E7-1F1F4",
    "flag-bq": "1F1E7-1F1F6",
    "flag-br": "1F1E7-1F1F7",
    "flag-bs": "1F1E7-1F1F8",
    "flag-bt": "1F1E7-1F1F9",
    "flag-bv": "1F1E7-1F1FB",
    "flag-bw": "1F1E7-1F1FC",
    "flag-by": "1F1E7-1F1FE",
    "flag-bz": "1F1E7-1F1FF",
    "flag-ca": "1F1E8-1F1E6",
    "flag-cc": "1F1E8-1F1E8",
    "flag-cd": "1F1E8-1F1E9",
    "flag-cf": "1F1E8-1F1EB",
    "flag-cg": "1F1E8-1F1EC",
    "flag-ch": "1F1E8-1F1ED",
    "flag-ci": "1F1E8-1F1EE",
    "flag-ck": "1F1E8-1F1F0",
    "flag-cl": "1F1E8-1F1F1",
    "flag-cm": "1F1E8-1F1F2",
    "cn": "1F1E8-1F1F3",
    "flag-cn": "1F1E8-1F1F3",
    "flag-co": "1F1E8-1F1F4",
    "flag-cp": "1F1E8-1F1F5",
    "flag-cr": "1F1E8-1F1F7",
    "flag-cu": "1F1E8-1F1FA",
    "flag-cv": "1F1E8-1F1FB",
    "flag-cw": "1F1E8-1F1FC",
    "flag-cx": "1F1E8-1F1FD",
    "flag-cy": "1F1E8-1F1FE",
    "flag-cz": "1F1E8-1F1FF",
    "de": "1F1E9-1F1EA",
    "flag-de": "1F1E9-1F1EA",
    "flag-dg": "1F1E9-1F1EC",
    "flag-dj": "1F1E9-1F1EF",
    "flag-dk": "1F1E9-1F1F0",
    "flag-dm": "1F1E9-1F1F2",
    "flag-do": "1F1E9-1F1F4",
    "flag-dz": "1F1E9-1F1FF",
    "flag-ea": "1F1EA-1F1E6",
    "flag-ec": "1F1EA-1F1E8",
    "flag-ee": "1F1EA-1F1EA",
    "flag-eg": "1F1EA-1F1EC",
    "flag-eh": "1F1EA-1F1ED",
    "flag-er": "1F1EA-1F1F7",
    "es": "1F1EA-1F1F8",
    "flag-es": "1F1EA-1F1F8",
    "flag-et": "1F1EA-1F1F9",
    "flag-eu": "1F1EA-1F1FA",
    "flag-fi": "1F1EB-1F1EE",
    "flag-fj": "1F1EB-1F1EF",
    "flag-fk": "1F1EB-1F1F0",
    "flag-fm": "1F1EB-1F1F2",
    "flag-fo": "1F1EB-1F1F4",
    "fr": "1F1EB-1F1F7",
    "flag-fr": "1F1EB-1F1F7",
    "flag-ga": "1F1EC-1F1E6",
    "gb": "1F1EC-1F1E7",
    "uk": "1F1EC-1F1E7",
    "flag-gb": "1F1EC-1F1E7",
    "flag-gd": "1F1EC-1F1E9",
    "flag-ge": "1F1EC-1F1EA",
    "flag-gf": "1F1EC-1F1EB",
    "flag-gg": "1F1EC-1F1EC",
    "flag-gh": "1F1EC-1F1ED",
    "flag-gi": "1F1EC-1F1EE",
    "flag-gl": "1F1EC-1F1F1",
    "flag-gm": "1F1EC-1F1F2",
    "flag-gn": "1F1EC-1F1F3",
    "flag-gp": "1F1EC-1F1F5",
    "flag-gq": "1F1EC-1F1F6",
    "flag-gr": "1F1EC-1F1F7",
    "flag-gs": "1F1EC-1F1F8",
    "flag-gt": "1F1EC-1F1F9",
    "flag-gu": "1F1EC-1F1FA",
    "flag-gw": "1F1EC-1F1FC",
    "flag-gy": "1F1EC-1F1FE",
    "flag-hk": "1F1ED-1F1F0",
    "flag-hm": "1F1ED-1F1F2",
    "flag-hn": "1F1ED-1F1F3",
    "flag-hr": "1F1ED-1F1F7",
    "flag-ht": "1F1ED-1F1F9",
    "flag-hu": "1F1ED-1F1FA",
    "flag-ic": "1F1EE-1F1E8",
    "flag-id": "1F1EE-1F1E9",
    "flag-ie": "1F1EE-1F1EA",
    "flag-il": "1F1EE-1F1F1",
    "flag-im": "1F1EE-1F1F2",
    "flag-in": "1F1EE-1F1F3",
    "flag-io": "1F1EE-1F1F4",
    "flag-iq": "1F1EE-1F1F6",
    "flag-ir": "1F1EE-1F1F7",
    "flag-is": "1F1EE-1F1F8",
    "it": "1F1EE-1F1F9",
    "flag-it": "1F1EE-1F1F9",
    "flag-je": "1F1EF-1F1EA",
    "flag-jm": "1F1EF-1F1F2",
    "flag-jo": "1F1EF-1F1F4",
    "jp": "1F1EF-1F1F5",
    "flag-jp": "1F1EF-1F1F5",
    "flag-ke": "1F1F0-1F1EA",
    "flag-kg": "1F1F0-1F1EC",
    "flag-kh": "1F1F0-1F1ED",
    "flag-ki": "1F1F0-1F1EE",
    "flag-km": "1F1F0-1F1F2",
    "flag-kn": "1F1F0-1F1F3",
    "flag-kp": "1F1F0-1F1F5",
    "kr": "1F1F0-1F1F7",
    "flag-kr": "1F1F0-1F1F7",
    "flag-kw": "1F1F0-1F1FC",
    "flag-ky": "1F1F0-1F1FE",
    "flag-kz": "1F1F0-1F1FF",
    "flag-la": "1F1F1-1F1E6",
    "flag-lb": "1F1F1-1F1E7",
    "flag-lc": "1F1F1-1F1E8",
    "flag-li": "1F1F1-1F1EE",
    "flag-lk": "1F1F1-1F1F0",
    "flag-lr": "1F1F1-1F1F7",
    "flag-ls": "1F1F1-1F1F8",
    "flag-lt": "1F1F1-1F1F9",
    "flag-lu": "1F1F1-1F1FA",
    "flag-lv": "1F1F1-1F1FB",
    "flag-ly": "1F1F1-1F1FE",
    "flag-ma": "1F1F2-1F1E6",
    "flag-mc": "1F1F2-1F1E8",
    "flag-md": "1F1F2-1F1E9",
    "flag-me": "1F1F2-1F1EA",
    "flag-mf": "1F1F2-1F1EB",
    "flag-mg": "1F1F2-1F1EC",
    "flag-mh": "1F1F2-1F1ED",
    "flag-mk": "1F1F2-1F1F0",
    "flag-ml": "1F1F2-1F1F1",
    "flag-mm": "1F1F2-1F1F2",
    "flag-mn": "1F1F2-1F1F3",
    "flag-mo": "1F1F2-1F1F4",
    "flag-mp": "1F1F2-1F1F5",
    "flag-mq": "1F1F2-1F1F6",
    "flag-mr": "1F1F2-1F1F7",
    "flag-ms": "1F1F2-1F1F8",
    "flag-mt": "1F1F2-1F1F9",
    "flag-mu": "1F1F2-1F1FA",
    "flag-mv": "1F1F2-1F1FB",
    "flag-mw": "1F1F2-1F1FC",
    "flag-mx": "1F1F2-1F1FD",
    "flag-my": "1F1F2-1F1FE",
    "flag-mz": "1F1F2-1F1FF",
    "flag-na": "1F1F3-1F1E6",
    "flag-nc": "1F1F3-1F1E8",
    "flag-ne": "1F1F3-1F1EA",
    "flag-nf": "1F1F3-1F1EB",
    "flag-ng": "1F1F3-1F1EC",
    "flag-ni": "1F1F3-1F1EE",
    "flag-nl": "1F1F3-1F1F1",
    "flag-no": "1F1F3-1F1F4",
    "flag-np": "1F1F3-1F1F5",
    "flag-nr": "1F1F3-1F1F7",
    "flag-nu": "1F1F3-1F1FA",
    "flag-nz": "1F1F3-1F1FF",
    "flag-om": "1F1F4-1F1F2",
    "flag-pa": "1F1F5-1F1E6",
    "flag-pe": "1F1F5-1F1EA",
    "flag-pf": "1F1F5-1F1EB",
    "flag-pg": "1F1F5-1F1EC",
    "flag-ph": "1F1F5-1F1ED",
    "flag-pk": "1F1F5-1F1F0",
    "flag-pl": "1F1F5-1F1F1",
    "flag-pm": "1F1F5-1F1F2",
    "flag-pn": "1F1F5-1F1F3",
    "flag-pr": "1F1F5-1F1F7",
    "flag-ps": "1F1F5-1F1F8",
    "flag-pt": "1F1F5-1F1F9",
    "flag-pw": "1F1F5-1F1FC",
    "flag-py": "1F1F5-1F1FE",
    "flag-qa": "1F1F6-1F1E6",
    "flag-re": "1F1F7-1F1EA",
    "flag-ro": "1F1F7-1F1F4",
    "flag-rs": "1F1F7-1F1F8",
    "ru": "1F1F7-1F1FA",
    "flag-ru": "1F1F7-1F1FA",
    "flag-rw": "1F1F7-1F1FC",
    "flag-sa": "1F1F8-1F1E6",
    "flag-sb": "1F1F8-1F1E7",
    "flag-sc": "1F1F8-1F1E8",
    "flag-sd": "1F1F8-1F1E9",
    "flag-se": "1F1F8-1F1EA",
    "flag-sg": "1F1F8-1F1EC",
    "flag-sh": "1F1F8-1F1ED",
    "flag-si": "1F1F8-1F1EE",
    "flag-sj": "1F1F8-1F1EF",
    "flag-sk": "1F1F8-1F1F0",
    "flag-sl": "1F1F8-1F1F1",
    "flag-sm": "1F1F8-1F1F2",
    "flag-sn": "1F1F8-1F1F3",
    "flag-so": "1F1F8-1F1F4",
    "flag-sr": "1F1F8-1F1F7",
    "flag-ss": "1F1F8-1F1F8",
    "flag-st": "1F1F8-1F1F9",
    "flag-sv": "1F1F8-1F1FB",
    "flag-sx": "1F1F8-1F1FD",
    "flag-sy": "1F1F8-1F1FE",
    "flag-sz": "1F1F8-1F1FF",
    "flag-ta": "1F1F9-1F1E6",
    "flag-tc": "1F1F9-1F1E8",
    "flag-td": "1F1F9-1F1E9",
    "flag-tf": "1F1F9-1F1EB",
    "flag-tg": "1F1F9-1F1EC",
    "flag-th": "1F1F9-1F1ED",
    "flag-tj": "1F1F9-1F1EF",
    "flag-tk": "1F1F9-1F1F0",
    "flag-tl": "1F1F9-1F1F1",
    "flag-tm": "1F1F9-1F1F2",
    "flag-tn": "1F1F9-1F1F3",
    "flag-to": "1F1F9-1F1F4",
    "flag-tr": "1F1F9-1F1F7",
    "flag-tt": "1F1F9-1F1F9",
    "flag-tv": "1F1F9-1F1FB",
    "flag-tw": "1F1F9-1F1FC",
    "flag-tz": "1F1F9-1F1FF",
    "flag-ua": "1F1FA-1F1E6",
    "flag-ug": "1F1FA-1F1EC",
    "flag-um": "1F1FA-1F1F2",
    "flag-un": "1F1FA-1F1F3",
    "us": "1F1FA-1F1F8",
    "flag-us": "1F1FA-1F1F8",
    "flag-uy": "1F1FA-1F1FE",
    "flag-uz": "1F1FA-1F1FF",
    "flag-va": "1F1FB-1F1E6",
    "flag-vc": "1F1FB-1F1E8",
    "flag-ve": "1F1FB-1F1EA",
    "flag-vg": "1F1FB-1F1EC",
    "flag-vi": "1F1FB-1F1EE",
    "flag-vn": "1F1FB-1F1F3",
    "flag-vu": "1F1FB-1F1FA",
    "flag-wf": "1F1FC-1F1EB",
    "flag-ws": "1F1FC-1F1F8",
    "flag-xk": "1F1FD-1F1F0",
    "flag-ye": "1F1FE-1F1EA",
    "flag-yt": "1F1FE-1F1F9",
    "flag-za": "1F1FF-1F1E6",
    "flag-zm": "1F1FF-1F1F2",
    "flag-zw": "1F1FF-1F1FC",
    "koko": "1F201",
    "sa": "1F202-FE0F",
    "u7121": "1F21A",
    "u6307": "1F22F",
    "u7981": "1F232",
    "u7a7a": "1F233",
    "u5408": "1F234",
    "u6e80": "1F235",
    "u6709": "1F236",
    "u6708": "1F237-FE0F",
    "u7533": "1F238",
    "u5272": "1F239",
    "u55b6": "1F23A",
    "ideograph_advantage": "1F250",
    "accept": "1F251",
    "cyclone": "1F300",
    "foggy": "1F301",
    "closed_umbrella": "1F302",
    "night_with_stars": "1F303",
    "sunrise_over_mountains": "1F304",
    "sunrise": "1F305",
    "city_sunset": "1F306",
    "city_sunrise": "1F307",
    "rainbow": "1F308",
    "bridge_at_night": "1F309",
    "ocean": "1F30A",
    "volcano": "1F30B",
    "milky_way": "1F30C",
    "earth_africa": "1F30D",
    "earth_americas": "1F30E",
    "earth_asia": "1F30F",
    "globe_with_meridians": "1F310",
    "new_moon": "1F311",
    "waxing_crescent_moon": "1F312",
    "first_quarter_moon": "1F313",
    "moon": "1F314",
    "waxing_gibbous_moon": "1F314",
    "full_moon": "1F315",
    "waning_gibbous_moon": "1F316",
    "last_quarter_moon": "1F317",
    "waning_crescent_moon": "1F318",
    "crescent_moon": "1F319",
    "new_moon_with_face": "1F31A",
    "first_quarter_moon_with_face": "1F31B",
    "last_quarter_moon_with_face": "1F31C",
    "full_moon_with_face": "1F31D",
    "sun_with_face": "1F31E",
    "star2": "1F31F",
    "stars": "1F320",
    "thermometer": "1F321-FE0F",
    "mostly_sunny": "1F324-FE0F",
    "sun_small_cloud": "1F324-FE0F",
    "barely_sunny": "1F325-FE0F",
    "sun_behind_cloud": "1F325-FE0F",
    "partly_sunny_rain": "1F326-FE0F",
    "sun_behind_rain_cloud": "1F326-FE0F",
    "rain_cloud": "1F327-FE0F",
    "snow_cloud": "1F328-FE0F",
    "lightning": "1F329-FE0F",
    "lightning_cloud": "1F329-FE0F",
    "tornado": "1F32A-FE0F",
    "tornado_cloud": "1F32A-FE0F",
    "fog": "1F32B-FE0F",
    "wind_blowing_face": "1F32C-FE0F",
    "hotdog": "1F32D",
    "taco": "1F32E",
    "burrito": "1F32F",
    "chestnut": "1F330",
    "seedling": "1F331",
    "evergreen_tree": "1F332",
    "deciduous_tree": "1F333",
    "palm_tree": "1F334",
    "cactus": "1F335",
    "hot_pepper": "1F336-FE0F",
    "tulip": "1F337",
    "cherry_blossom": "1F338",
    "rose": "1F339",
    "hibiscus": "1F33A",
    "sunflower": "1F33B",
    "blossom": "1F33C",
    "corn": "1F33D",
    "ear_of_rice": "1F33E",
    "herb": "1F33F",
    "four_leaf_clover": "1F340",
    "maple_leaf": "1F341",
    "fallen_leaf": "1F342",
    "leaves": "1F343",
    "mushroom": "1F344",
    "tomato": "1F345",
    "eggplant": "1F346",
    "grapes": "1F347",
    "melon": "1F348",
    "watermelon": "1F349",
    "tangerine": "1F34A",
    "lemon": "1F34B",
    "banana": "1F34C",
    "pineapple": "1F34D",
    "apple": "1F34E",
    "green_apple": "1F34F",
    "pear": "1F350",
    "peach": "1F351",
    "cherries": "1F352",
    "strawberry": "1F353",
    "hamburger": "1F354",
    "pizza": "1F355",
    "meat_on_bone": "1F356",
    "poultry_leg": "1F357",
    "rice_cracker": "1F358",
    "rice_ball": "1F359",
    "rice": "1F35A",
    "curry": "1F35B",
    "ramen": "1F35C",
    "spaghetti": "1F35D",
    "bread": "1F35E",
    "fries": "1F35F",
    "sweet_potato": "1F360",
    "dango": "1F361",
    "oden": "1F362",
    "sushi": "1F363",
    "fried_shrimp": "1F364",
    "fish_cake": "1F365",
    "icecream": "1F366",
    "shaved_ice": "1F367",
    "ice_cream": "1F368",
    "doughnut": "1F369",
    "cookie": "1F36A",
    "chocolate_bar": "1F36B",
    "candy": "1F36C",
    "lollipop": "1F36D",
    "custard": "1F36E",
    "honey_pot": "1F36F",
    "cake": "1F370",
    "bento": "1F371",
    "stew": "1F372",
    "fried_egg": "1F373",
    "cooking": "1F373",
    "fork_and_knife": "1F374",
    "tea": "1F375",
    "sake": "1F376",
    "wine_glass": "1F377",
    "cocktail": "1F378",
    "tropical_drink": "1F379",
    "beer": "1F37A",
    "beers": "1F37B",
    "baby_bottle": "1F37C",
    "knife_fork_plate": "1F37D-FE0F",
    "champagne": "1F37E",
    "popcorn": "1F37F",
    "ribbon": "1F380",
    "gift": "1F381",
    "birthday": "1F382",
    "jack_o_lantern": "1F383",
    "christmas_tree": "1F384",
    "santa": "1F385",
    "fireworks": "1F386",
    "sparkler": "1F387",
    "balloon": "1F388",
    "tada": "1F389",
    "confetti_ball": "1F38A",
    "tanabata_tree": "1F38B",
    "crossed_flags": "1F38C",
    "bamboo": "1F38D",
    "dolls": "1F38E",
    "flags": "1F38F",
    "wind_chime": "1F390",
    "rice_scene": "1F391",
    "school_satchel": "1F392",
    "mortar_board": "1F393",
    "medal": "1F396-FE0F",
    "reminder_ribbon": "1F397-FE0F",
    "studio_microphone": "1F399-FE0F",
    "level_slider": "1F39A-FE0F",
    "control_knobs": "1F39B-FE0F",
    "film_frames": "1F39E-FE0F",
    "admission_tickets": "1F39F-FE0F",
    "carousel_horse": "1F3A0",
    "ferris_wheel": "1F3A1",
    "roller_coaster": "1F3A2",
    "fishing_pole_and_fish": "1F3A3",
    "microphone": "1F3A4",
    "movie_camera": "1F3A5",
    "cinema": "1F3A6",
    "headphones": "1F3A7",
    "art": "1F3A8",
    "tophat": "1F3A9",
    "circus_tent": "1F3AA",
    "ticket": "1F3AB",
    "clapper": "1F3AC",
    "performing_arts": "1F3AD",
    "video_game": "1F3AE",
    "dart": "1F3AF",
    "slot_machine": "1F3B0",
    "8ball": "1F3B1",
    "game_die": "1F3B2",
    "bowling": "1F3B3",
    "flower_playing_cards": "1F3B4",
    "musical_note": "1F3B5",
    "notes": "1F3B6",
    "saxophone": "1F3B7",
    "guitar": "1F3B8",
    "musical_keyboard": "1F3B9",
    "trumpet": "1F3BA",
    "violin": "1F3BB",
    "musical_score": "1F3BC",
    "running_shirt_with_sash": "1F3BD",
    "tennis": "1F3BE",
    "ski": "1F3BF",
    "basketball": "1F3C0",
    "checkered_flag": "1F3C1",
    "snowboarder": "1F3C2",
    "woman-running": "1F3C3-200D-2640-FE0F",
    "man-running": "1F3C3-200D-2642-FE0F",
    "runner": "1F3C3",
    "running": "1F3C3",
    "woman-surfing": "1F3C4-200D-2640-FE0F",
    "man-surfing": "1F3C4-200D-2642-FE0F",
    "surfer": "1F3C4",
    "sports_medal": "1F3C5",
    "trophy": "1F3C6",
    "horse_racing": "1F3C7",
    "football": "1F3C8",
    "rugby_football": "1F3C9",
    "woman-swimming": "1F3CA-200D-2640-FE0F",
    "man-swimming": "1F3CA-200D-2642-FE0F",
    "swimmer": "1F3CA",
    "woman-lifting-weights": "1F3CB-FE0F-200D-2640-FE0F",
    "man-lifting-weights": "1F3CB-FE0F-200D-2642-FE0F",
    "weight_lifter": "1F3CB-FE0F",
    "woman-golfing": "1F3CC-FE0F-200D-2640-FE0F",
    "man-golfing": "1F3CC-FE0F-200D-2642-FE0F",
    "golfer": "1F3CC-FE0F",
    "racing_motorcycle": "1F3CD-FE0F",
    "racing_car": "1F3CE-FE0F",
    "cricket_bat_and_ball": "1F3CF",
    "volleyball": "1F3D0",
    "field_hockey_stick_and_ball": "1F3D1",
    "ice_hockey_stick_and_puck": "1F3D2",
    "table_tennis_paddle_and_ball": "1F3D3",
    "snow_capped_mountain": "1F3D4-FE0F",
    "camping": "1F3D5-FE0F",
    "beach_with_umbrella": "1F3D6-FE0F",
    "building_construction": "1F3D7-FE0F",
    "house_buildings": "1F3D8-FE0F",
    "cityscape": "1F3D9-FE0F",
    "derelict_house_building": "1F3DA-FE0F",
    "classical_building": "1F3DB-FE0F",
    "desert": "1F3DC-FE0F",
    "desert_island": "1F3DD-FE0F",
    "national_park": "1F3DE-FE0F",
    "stadium": "1F3DF-FE0F",
    "house": "1F3E0",
    "house_with_garden": "1F3E1",
    "office": "1F3E2",
    "post_office": "1F3E3",
    "european_post_office": "1F3E4",
    "hospital": "1F3E5",
    "bank": "1F3E6",
    "atm": "1F3E7",
    "hotel": "1F3E8",
    "love_hotel": "1F3E9",
    "convenience_store": "1F3EA",
    "school": "1F3EB",
    "department_store": "1F3EC",
    "factory": "1F3ED",
    "izakaya_lantern": "1F3EE",
    "lantern": "1F3EE",
    "japanese_castle": "1F3EF",
    "european_castle": "1F3F0",
    "rainbow-flag": "1F3F3-FE0F-200D-1F308",
    "transgender_flag": "1F3F3-FE0F-200D-26A7-FE0F",
    "waving_white_flag": "1F3F3-FE0F",
    "pirate_flag": "1F3F4-200D-2620-FE0F",
    "flag-england": "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F",
    "flag-scotland": "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F",
    "flag-wales": "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F",
    "waving_black_flag": "1F3F4",
    "rosette": "1F3F5-FE0F",
    "label": "1F3F7-FE0F",
    "badminton_racquet_and_shuttlecock": "1F3F8",
    "bow_and_arrow": "1F3F9",
    "amphora": "1F3FA",
    "skin-tone-2": "1F3FB",
    "skin-tone-3": "1F3FC",
    "skin-tone-4": "1F3FD",
    "skin-tone-5": "1F3FE",
    "skin-tone-6": "1F3FF",
    "rat": "1F400",
    "mouse2": "1F401",
    "ox": "1F402",
    "water_buffalo": "1F403",
    "cow2": "1F404",
    "tiger2": "1F405",
    "leopard": "1F406",
    "rabbit2": "1F407",
    "black_cat": "1F408-200D-2B1B",
    "cat2": "1F408",
    "dragon": "1F409",
    "crocodile": "1F40A",
    "whale2": "1F40B",
    "snail": "1F40C",
    "snake": "1F40D",
    "racehorse": "1F40E",
    "ram": "1F40F",
    "goat": "1F410",
    "sheep": "1F411",
    "monkey": "1F412",
    "rooster": "1F413",
    "chicken": "1F414",
    "service_dog": "1F415-200D-1F9BA",
    "dog2": "1F415",
    "pig2": "1F416",
    "boar": "1F417",
    "elephant": "1F418",
    "octopus": "1F419",
    "shell": "1F41A",
    "bug": "1F41B",
    "ant": "1F41C",
    "bee": "1F41D",
    "honeybee": "1F41D",
    "beetle": "1FAB2",
    "fish": "1F41F",
    "tropical_fish": "1F420",
    "blowfish": "1F421",
    "turtle": "1F422",
    "hatching_chick": "1F423",
    "baby_chick": "1F424",
    "hatched_chick": "1F425",
    "bird": "1F426",
    "penguin": "1F427",
    "koala": "1F428",
    "poodle": "1F429",
    "dromedary_camel": "1F42A",
    "camel": "1F42B",
    "dolphin": "1F42C",
    "flipper": "1F42C",
    "mouse": "1F42D",
    "cow": "1F42E",
    "tiger": "1F42F",
    "rabbit": "1F430",
    "cat": "1F431",
    "dragon_face": "1F432",
    "whale": "1F433",
    "horse": "1F434",
    "monkey_face": "1F435",
    "dog": "1F436",
    "pig": "1F437",
    "frog": "1F438",
    "hamster": "1F439",
    "wolf": "1F43A",
    "polar_bear": "1F43B-200D-2744-FE0F",
    "bear": "1F43B",
    "panda_face": "1F43C",
    "pig_nose": "1F43D",
    "feet": "1F43E",
    "paw_prints": "1F43E",
    "chipmunk": "1F43F-FE0F",
    "eyes": "1F440",
    "eye-in-speech-bubble": "1F441-FE0F-200D-1F5E8-FE0F",
    "eye": "1F441-FE0F",
    "ear": "1F442",
    "nose": "1F443",
    "lips": "1F444",
    "tongue": "1F445",
    "point_up_2": "1F446",
    "point_down": "1F447",
    "point_left": "1F448",
    "point_right": "1F449",
    "facepunch": "1F44A",
    "punch": "1F44A",
    "wave": "1F44B",
    "ok_hand": "1F44C",
    "+1": "1F44D",
    "thumbsup": "1F44D",
    "-1": "1F44E",
    "thumbsdown": "1F44E",
    "clap": "1F44F",
    "open_hands": "1F450",
    "crown": "1F451",
    "womans_hat": "1F452",
    "eyeglasses": "1F453",
    "necktie": "1F454",
    "shirt": "1F455",
    "tshirt": "1F455",
    "jeans": "1F456",
    "dress": "1F457",
    "kimono": "1F458",
    "bikini": "1F459",
    "womans_clothes": "1F45A",
    "purse": "1F45B",
    "handbag": "1F45C",
    "pouch": "1F45D",
    "mans_shoe": "1F45E",
    "shoe": "1F45E",
    "athletic_shoe": "1F45F",
    "high_heel": "1F460",
    "sandal": "1F461",
    "boot": "1F462",
    "footprints": "1F463",
    "bust_in_silhouette": "1F464",
    "busts_in_silhouette": "1F465",
    "boy": "1F466",
    "girl": "1F467",
    "male-farmer": "1F468-200D-1F33E",
    "male-cook": "1F468-200D-1F373",
    "man_feeding_baby": "1F468-200D-1F37C",
    "male-student": "1F468-200D-1F393",
    "male-singer": "1F468-200D-1F3A4",
    "male-artist": "1F468-200D-1F3A8",
    "male-teacher": "1F468-200D-1F3EB",
    "male-factory-worker": "1F468-200D-1F3ED",
    "man-boy-boy": "1F468-200D-1F466-200D-1F466",
    "man-boy": "1F468-200D-1F466",
    "man-girl-boy": "1F468-200D-1F467-200D-1F466",
    "man-girl-girl": "1F468-200D-1F467-200D-1F467",
    "man-girl": "1F468-200D-1F467",
    "man-man-boy": "1F468-200D-1F468-200D-1F466",
    "man-man-boy-boy": "1F468-200D-1F468-200D-1F466-200D-1F466",
    "man-man-girl": "1F468-200D-1F468-200D-1F467",
    "man-man-girl-boy": "1F468-200D-1F468-200D-1F467-200D-1F466",
    "man-man-girl-girl": "1F468-200D-1F468-200D-1F467-200D-1F467",
    "man-woman-boy": "1F46A",
    "family": "1F46A",
    "man-woman-boy-boy": "1F468-200D-1F469-200D-1F466-200D-1F466",
    "man-woman-girl": "1F468-200D-1F469-200D-1F467",
    "man-woman-girl-boy": "1F468-200D-1F469-200D-1F467-200D-1F466",
    "man-woman-girl-girl": "1F468-200D-1F469-200D-1F467-200D-1F467",
    "male-technologist": "1F468-200D-1F4BB",
    "male-office-worker": "1F468-200D-1F4BC",
    "male-mechanic": "1F468-200D-1F527",
    "male-scientist": "1F468-200D-1F52C",
    "male-astronaut": "1F468-200D-1F680",
    "male-firefighter": "1F468-200D-1F692",
    "man_with_probing_cane": "1F468-200D-1F9AF",
    "red_haired_man": "1F468-200D-1F9B0",
    "curly_haired_man": "1F468-200D-1F9B1",
    "bald_man": "1F468-200D-1F9B2",
    "white_haired_man": "1F468-200D-1F9B3",
    "man_in_motorized_wheelchair": "1F468-200D-1F9BC",
    "man_in_manual_wheelchair": "1F468-200D-1F9BD",
    "male-doctor": "1F468-200D-2695-FE0F",
    "male-judge": "1F468-200D-2696-FE0F",
    "male-pilot": "1F468-200D-2708-FE0F",
    "man-heart-man": "1F468-200D-2764-FE0F-200D-1F468",
    "man-kiss-man": "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468",
    "man": "1F468",
    "female-farmer": "1F469-200D-1F33E",
    "female-cook": "1F469-200D-1F373",
    "woman_feeding_baby": "1F469-200D-1F37C",
    "female-student": "1F469-200D-1F393",
    "female-singer": "1F469-200D-1F3A4",
    "female-artist": "1F469-200D-1F3A8",
    "female-teacher": "1F469-200D-1F3EB",
    "female-factory-worker": "1F469-200D-1F3ED",
    "woman-boy-boy": "1F469-200D-1F466-200D-1F466",
    "woman-boy": "1F469-200D-1F466",
    "woman-girl-boy": "1F469-200D-1F467-200D-1F466",
    "woman-girl-girl": "1F469-200D-1F467-200D-1F467",
    "woman-girl": "1F469-200D-1F467",
    "woman-woman-boy": "1F469-200D-1F469-200D-1F466",
    "woman-woman-boy-boy": "1F469-200D-1F469-200D-1F466-200D-1F466",
    "woman-woman-girl": "1F469-200D-1F469-200D-1F467",
    "woman-woman-girl-boy": "1F469-200D-1F469-200D-1F467-200D-1F466",
    "woman-woman-girl-girl": "1F469-200D-1F469-200D-1F467-200D-1F467",
    "female-technologist": "1F469-200D-1F4BB",
    "female-office-worker": "1F469-200D-1F4BC",
    "female-mechanic": "1F469-200D-1F527",
    "female-scientist": "1F469-200D-1F52C",
    "female-astronaut": "1F469-200D-1F680",
    "female-firefighter": "1F469-200D-1F692",
    "woman_with_probing_cane": "1F469-200D-1F9AF",
    "red_haired_woman": "1F469-200D-1F9B0",
    "curly_haired_woman": "1F469-200D-1F9B1",
    "bald_woman": "1F469-200D-1F9B2",
    "white_haired_woman": "1F469-200D-1F9B3",
    "woman_in_motorized_wheelchair": "1F469-200D-1F9BC",
    "woman_in_manual_wheelchair": "1F469-200D-1F9BD",
    "female-doctor": "1F469-200D-2695-FE0F",
    "female-judge": "1F469-200D-2696-FE0F",
    "female-pilot": "1F469-200D-2708-FE0F",
    "woman-heart-man": "1F469-200D-2764-FE0F-200D-1F468",
    "woman-heart-woman": "1F469-200D-2764-FE0F-200D-1F469",
    "woman-kiss-man": "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468",
    "woman-kiss-woman": "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469",
    "woman": "1F469",
    "couple": "1F46B",
    "man_and_woman_holding_hands": "1F46B",
    "woman_and_man_holding_hands": "1F46B",
    "two_men_holding_hands": "1F46C",
    "men_holding_hands": "1F46C",
    "two_women_holding_hands": "1F46D",
    "women_holding_hands": "1F46D",
    "female-police-officer": "1F46E-200D-2640-FE0F",
    "male-police-officer": "1F46E-200D-2642-FE0F",
    "cop": "1F46E",
    "woman-with-bunny-ears-partying": "1F46F-200D-2640-FE0F",
    "man-with-bunny-ears-partying": "1F46F-200D-2642-FE0F",
    "dancers": "1F46F",
    "woman_with_veil": "1F470-200D-2640-FE0F",
    "man_with_veil": "1F470-200D-2642-FE0F",
    "bride_with_veil": "1F470",
    "blond-haired-woman": "1F471-200D-2640-FE0F",
    "blond-haired-man": "1F471-200D-2642-FE0F",
    "person_with_blond_hair": "1F471",
    "man_with_gua_pi_mao": "1F472",
    "woman-wearing-turban": "1F473-200D-2640-FE0F",
    "man-wearing-turban": "1F473-200D-2642-FE0F",
    "man_with_turban": "1F473",
    "older_man": "1F474",
    "older_woman": "1F475",
    "baby": "1F476",
    "female-construction-worker": "1F477-200D-2640-FE0F",
    "male-construction-worker": "1F477-200D-2642-FE0F",
    "construction_worker": "1F477",
    "princess": "1F478",
    "japanese_ogre": "1F479",
    "japanese_goblin": "1F47A",
    "ghost": "1F47B",
    "angel": "1F47C",
    "alien": "1F47D",
    "space_invader": "1F47E",
    "imp": "1F47F",
    "skull": "1F480",
    "woman-tipping-hand": "1F481-200D-2640-FE0F",
    "man-tipping-hand": "1F481-200D-2642-FE0F",
    "information_desk_person": "1F481",
    "female-guard": "1F482-200D-2640-FE0F",
    "male-guard": "1F482-200D-2642-FE0F",
    "guardsman": "1F482",
    "dancer": "1F483",
    "lipstick": "1F484",
    "nail_care": "1F485",
    "woman-getting-massage": "1F486-200D-2640-FE0F",
    "man-getting-massage": "1F486-200D-2642-FE0F",
    "massage": "1F486",
    "woman-getting-haircut": "1F487-200D-2640-FE0F",
    "man-getting-haircut": "1F487-200D-2642-FE0F",
    "haircut": "1F487",
    "barber": "1F488",
    "syringe": "1F489",
    "pill": "1F48A",
    "kiss": "1F48B",
    "love_letter": "1F48C",
    "ring": "1F48D",
    "gem": "1F48E",
    "couplekiss": "1F48F",
    "bouquet": "1F490",
    "couple_with_heart": "1F491",
    "wedding": "1F492",
    "heartbeat": "1F493",
    "broken_heart": "1F494",
    "two_hearts": "1F495",
    "sparkling_heart": "1F496",
    "heartpulse": "1F497",
    "cupid": "1F498",
    "blue_heart": "1F499",
    "green_heart": "1F49A",
    "yellow_heart": "1F49B",
    "purple_heart": "1F49C",
    "gift_heart": "1F49D",
    "revolving_hearts": "1F49E",
    "heart_decoration": "1F49F",
    "diamond_shape_with_a_dot_inside": "1F4A0",
    "bulb": "1F4A1",
    "anger": "1F4A2",
    "bomb": "1F4A3",
    "zzz": "1F4A4",
    "boom": "1F4A5",
    "collision": "1F4A5",
    "sweat_drops": "1F4A6",
    "droplet": "1F4A7",
    "dash": "1F4A8",
    "hankey": "1F4A9",
    "poop": "1F4A9",
    "shit": "1F4A9",
    "muscle": "1F4AA",
    "dizzy": "1F4AB",
    "speech_balloon": "1F4AC",
    "thought_balloon": "1F4AD",
    "white_flower": "1F4AE",
    "moneybag": "1F4B0",
    "currency_exchange": "1F4B1",
    "heavy_dollar_sign": "1F4B2",
    "credit_card": "1F4B3",
    "yen": "1F4B4",
    "dollar": "1F4B5",
    "euro": "1F4B6",
    "pound": "1F4B7",
    "money_with_wings": "1F4B8",
    "chart": "1F4B9",
    "seat": "1F4BA",
    "computer": "1F4BB",
    "briefcase": "1F4BC",
    "minidisc": "1F4BD",
    "floppy_disk": "1F4BE",
    "cd": "1F4BF",
    "dvd": "1F4C0",
    "file_folder": "1F4C1",
    "open_file_folder": "1F4C2",
    "page_with_curl": "1F4C3",
    "page_facing_up": "1F4C4",
    "date": "1F4C5",
    "calendar": "1F4C6",
    "card_index": "1F4C7",
    "chart_with_upwards_trend": "1F4C8",
    "chart_with_downwards_trend": "1F4C9",
    "bar_chart": "1F4CA",
    "clipboard": "1F4CB",
    "pushpin": "1F4CC",
    "round_pushpin": "1F4CD",
    "paperclip": "1F4CE",
    "straight_ruler": "1F4CF",
    "triangular_ruler": "1F4D0",
    "bookmark_tabs": "1F4D1",
    "ledger": "1F4D2",
    "notebook": "1F4D3",
    "notebook_with_decorative_cover": "1F4D4",
    "closed_book": "1F4D5",
    "book": "1F4D6",
    "open_book": "1F4D6",
    "green_book": "1F4D7",
    "blue_book": "1F4D8",
    "orange_book": "1F4D9",
    "books": "1F4DA",
    "name_badge": "1F4DB",
    "scroll": "1F4DC",
    "memo": "1F4DD",
    "pencil": "1F4DD",
    "telephone_receiver": "1F4DE",
    "pager": "1F4DF",
    "fax": "1F4E0",
    "satellite_antenna": "1F4E1",
    "loudspeaker": "1F4E2",
    "mega": "1F4E3",
    "outbox_tray": "1F4E4",
    "inbox_tray": "1F4E5",
    "package": "1F4E6",
    "e-mail": "1F4E7",
    "incoming_envelope": "1F4E8",
    "envelope_with_arrow": "1F4E9",
    "mailbox_closed": "1F4EA",
    "mailbox": "1F4EB",
    "mailbox_with_mail": "1F4EC",
    "mailbox_with_no_mail": "1F4ED",
    "postbox": "1F4EE",
    "postal_horn": "1F4EF",
    "newspaper": "1F4F0",
    "iphone": "1F4F1",
    "calling": "1F4F2",
    "vibration_mode": "1F4F3",
    "mobile_phone_off": "1F4F4",
    "no_mobile_phones": "1F4F5",
    "signal_strength": "1F4F6",
    "camera": "1F4F7",
    "camera_with_flash": "1F4F8",
    "video_camera": "1F4F9",
    "tv": "1F4FA",
    "radio": "1F4FB",
    "vhs": "1F4FC",
    "film_projector": "1F4FD-FE0F",
    "prayer_beads": "1F4FF",
    "twisted_rightwards_arrows": "1F500",
    "repeat": "1F501",
    "repeat_one": "1F502",
    "arrows_clockwise": "1F503",
    "arrows_counterclockwise": "1F504",
    "low_brightness": "1F505",
    "high_brightness": "1F506",
    "mute": "1F507",
    "speaker": "1F508",
    "sound": "1F509",
    "loud_sound": "1F50A",
    "battery": "1F50B",
    "electric_plug": "1F50C",
    "mag": "1F50D",
    "mag_right": "1F50E",
    "lock_with_ink_pen": "1F50F",
    "closed_lock_with_key": "1F510",
    "key": "1F511",
    "lock": "1F512",
    "unlock": "1F513",
    "bell": "1F514",
    "no_bell": "1F515",
    "bookmark": "1F516",
    "link": "1F517",
    "radio_button": "1F518",
    "back": "1F519",
    "end": "1F51A",
    "on": "1F51B",
    "soon": "1F51C",
    "top": "1F51D",
    "underage": "1F51E",
    "keycap_ten": "1F51F",
    "capital_abcd": "1F520",
    "abcd": "1F521",
    "symbols": "1F523",
    "abc": "1F524",
    "fire": "1F525",
    "flashlight": "1F526",
    "wrench": "1F527",
    "hammer": "1F528",
    "nut_and_bolt": "1F529",
    "hocho": "1F52A",
    "knife": "1F52A",
    "gun": "1F52B",
    "microscope": "1F52C",
    "telescope": "1F52D",
    "crystal_ball": "1F52E",
    "six_pointed_star": "1F52F",
    "beginner": "1F530",
    "trident": "1F531",
    "black_square_button": "1F532",
    "white_square_button": "1F533",
    "red_circle": "1F534",
    "large_blue_circle": "1F535",
    "large_orange_diamond": "1F536",
    "large_blue_diamond": "1F537",
    "small_orange_diamond": "1F538",
    "small_blue_diamond": "1F539",
    "small_red_triangle": "1F53A",
    "small_red_triangle_down": "1F53B",
    "arrow_up_small": "1F53C",
    "arrow_down_small": "1F53D",
    "om_symbol": "1F549-FE0F",
    "dove_of_peace": "1F54A-FE0F",
    "kaaba": "1F54B",
    "mosque": "1F54C",
    "synagogue": "1F54D",
    "menorah_with_nine_branches": "1F54E",
    "clock1": "1F550",
    "clock2": "1F551",
    "clock3": "1F552",
    "clock4": "1F553",
    "clock5": "1F554",
    "clock6": "1F555",
    "clock7": "1F556",
    "clock8": "1F557",
    "clock9": "1F558",
    "clock10": "1F559",
    "clock11": "1F55A",
    "clock12": "1F55B",
    "clock130": "1F55C",
    "clock230": "1F55D",
    "clock330": "1F55E",
    "clock430": "1F55F",
    "clock530": "1F560",
    "clock630": "1F561",
    "clock730": "1F562",
    "clock830": "1F563",
    "clock930": "1F564",
    "clock1030": "1F565",
    "clock1130": "1F566",
    "clock1230": "1F567",
    "candle": "1F56F-FE0F",
    "mantelpiece_clock": "1F570-FE0F",
    "hole": "1F573-FE0F",
    "man_in_business_suit_levitating": "1F574-FE0F",
    "female-detective": "1F575-FE0F-200D-2640-FE0F",
    "male-detective": "1F575-FE0F-200D-2642-FE0F",
    "sleuth_or_spy": "1F575-FE0F",
    "dark_sunglasses": "1F576-FE0F",
    "spider": "1F577-FE0F",
    "spider_web": "1F578-FE0F",
    "joystick": "1F579-FE0F",
    "man_dancing": "1F57A",
    "linked_paperclips": "1F587-FE0F",
    "lower_left_ballpoint_pen": "1F58A-FE0F",
    "lower_left_fountain_pen": "1F58B-FE0F",
    "lower_left_paintbrush": "1F58C-FE0F",
    "lower_left_crayon": "1F58D-FE0F",
    "raised_hand_with_fingers_splayed": "1F590-FE0F",
    "middle_finger": "1F595",
    "reversed_hand_with_middle_finger_extended": "1F595",
    "spock-hand": "1F596",
    "black_heart": "1F5A4",
    "desktop_computer": "1F5A5-FE0F",
    "printer": "1F5A8-FE0F",
    "three_button_mouse": "1F5B1-FE0F",
    "trackball": "1F5B2-FE0F",
    "frame_with_picture": "1F5BC-FE0F",
    "card_index_dividers": "1F5C2-FE0F",
    "card_file_box": "1F5C3-FE0F",
    "file_cabinet": "1F5C4-FE0F",
    "wastebasket": "1F5D1-FE0F",
    "spiral_note_pad": "1F5D2-FE0F",
    "spiral_calendar_pad": "1F5D3-FE0F",
    "compression": "1F5DC-FE0F",
    "old_key": "1F5DD-FE0F",
    "rolled_up_newspaper": "1F5DE-FE0F",
    "dagger_knife": "1F5E1-FE0F",
    "speaking_head_in_silhouette": "1F5E3-FE0F",
    "left_speech_bubble": "1F5E8-FE0F",
    "right_anger_bubble": "1F5EF-FE0F",
    "ballot_box_with_ballot": "1F5F3-FE0F",
    "world_map": "1F5FA-FE0F",
    "mount_fuji": "1F5FB",
    "tokyo_tower": "1F5FC",
    "statue_of_liberty": "1F5FD",
    "japan": "1F5FE",
    "moyai": "1F5FF",
    "grinning": "1F600",
    "grin": "1F601",
    "joy": "1F602",
    "smiley": "1F603",
    "smile": "1F604",
    "sweat_smile": "1F605",
    "laughing": "1F606",
    "satisfied": "1F606",
    "innocent": "1F607",
    "smiling_imp": "1F608",
    "wink": "1F609",
    "blush": "1F60A",
    "yum": "1F60B",
    "relieved": "1F60C",
    "heart_eyes": "1F60D",
    "sunglasses": "1F60E",
    "smirk": "1F60F",
    "neutral_face": "1F610",
    "expressionless": "1F611",
    "unamused": "1F612",
    "sweat": "1F613",
    "pensive": "1F614",
    "confused": "1F615",
    "confounded": "1F616",
    "kissing": "1F617",
    "kissing_heart": "1F618",
    "kissing_smiling_eyes": "1F619",
    "kissing_closed_eyes": "1F61A",
    "stuck_out_tongue": "1F61B",
    "stuck_out_tongue_winking_eye": "1F61C",
    "stuck_out_tongue_closed_eyes": "1F61D",
    "disappointed": "1F61E",
    "worried": "1F61F",
    "angry": "1F620",
    "rage": "1F621",
    "cry": "1F622",
    "persevere": "1F623",
    "triumph": "1F624",
    "disappointed_relieved": "1F625",
    "frowning": "1F626",
    "anguished": "1F627",
    "fearful": "1F628",
    "weary": "1F629",
    "sleepy": "1F62A",
    "tired_face": "1F62B",
    "grimacing": "1F62C",
    "sob": "1F62D",
    "open_mouth": "1F62E",
    "hushed": "1F62F",
    "cold_sweat": "1F630",
    "scream": "1F631",
    "astonished": "1F632",
    "flushed": "1F633",
    "sleeping": "1F634",
    "dizzy_face": "1F635",
    "no_mouth": "1F636",
    "mask": "1F637",
    "smile_cat": "1F638",
    "joy_cat": "1F639",
    "smiley_cat": "1F63A",
    "heart_eyes_cat": "1F63B",
    "smirk_cat": "1F63C",
    "kissing_cat": "1F63D",
    "pouting_cat": "1F63E",
    "crying_cat_face": "1F63F",
    "scream_cat": "1F640",
    "slightly_frowning_face": "1F641",
    "slightly_smiling_face": "1F642",
    "upside_down_face": "1F643",
    "face_with_rolling_eyes": "1F644",
    "woman-gesturing-no": "1F645-200D-2640-FE0F",
    "man-gesturing-no": "1F645-200D-2642-FE0F",
    "no_good": "1F645",
    "woman-gesturing-ok": "1F646-200D-2640-FE0F",
    "man-gesturing-ok": "1F646-200D-2642-FE0F",
    "ok_woman": "1F646",
    "woman-bowing": "1F647-200D-2640-FE0F",
    "man-bowing": "1F647-200D-2642-FE0F",
    "bow": "1F647",
    "see_no_evil": "1F648",
    "hear_no_evil": "1F649",
    "speak_no_evil": "1F64A",
    "woman-raising-hand": "1F64B-200D-2640-FE0F",
    "man-raising-hand": "1F64B-200D-2642-FE0F",
    "raising_hand": "1F64B",
    "raised_hands": "1F64C",
    "woman-frowning": "1F64D-200D-2640-FE0F",
    "man-frowning": "1F64D-200D-2642-FE0F",
    "person_frowning": "1F64D",
    "woman-pouting": "1F64E-200D-2640-FE0F",
    "man-pouting": "1F64E-200D-2642-FE0F",
    "person_with_pouting_face": "1F64E",
    "pray": "1F64F",
    "rocket": "1F680",
    "helicopter": "1F681",
    "steam_locomotive": "1F682",
    "railway_car": "1F683",
    "bullettrain_side": "1F684",
    "bullettrain_front": "1F685",
    "train2": "1F686",
    "metro": "1F687",
    "light_rail": "1F688",
    "station": "1F689",
    "tram": "1F68A",
    "train": "1F68B",
    "bus": "1F68C",
    "oncoming_bus": "1F68D",
    "trolleybus": "1F68E",
    "busstop": "1F68F",
    "minibus": "1F690",
    "ambulance": "1F691",
    "fire_engine": "1F692",
    "police_car": "1F693",
    "oncoming_police_car": "1F694",
    "taxi": "1F695",
    "oncoming_taxi": "1F696",
    "car": "1F697",
    "red_car": "1F697",
    "oncoming_automobile": "1F698",
    "blue_car": "1F699",
    "truck": "1F69A",
    "articulated_lorry": "1F69B",
    "tractor": "1F69C",
    "monorail": "1F69D",
    "mountain_railway": "1F69E",
    "suspension_railway": "1F69F",
    "mountain_cableway": "1F6A0",
    "aerial_tramway": "1F6A1",
    "ship": "1F6A2",
    "woman-rowing-boat": "1F6A3-200D-2640-FE0F",
    "man-rowing-boat": "1F6A3-200D-2642-FE0F",
    "rowboat": "1F6A3",
    "speedboat": "1F6A4",
    "traffic_light": "1F6A5",
    "vertical_traffic_light": "1F6A6",
    "construction": "1F6A7",
    "rotating_light": "1F6A8",
    "triangular_flag_on_post": "1F6A9",
    "door": "1F6AA",
    "no_entry_sign": "1F6AB",
    "smoking": "1F6AC",
    "no_smoking": "1F6AD",
    "put_litter_in_its_place": "1F6AE",
    "do_not_litter": "1F6AF",
    "potable_water": "1F6B0",
    "non-potable_water": "1F6B1",
    "bike": "1F6B2",
    "no_bicycles": "1F6B3",
    "woman-biking": "1F6B4-200D-2640-FE0F",
    "man-biking": "1F6B4-200D-2642-FE0F",
    "bicyclist": "1F6B4",
    "woman-mountain-biking": "1F6B5-200D-2640-FE0F",
    "man-mountain-biking": "1F6B5-200D-2642-FE0F",
    "mountain_bicyclist": "1F6B5",
    "woman-walking": "1F6B6-200D-2640-FE0F",
    "man-walking": "1F6B6-200D-2642-FE0F",
    "walking": "1F6B6",
    "no_pedestrians": "1F6B7",
    "children_crossing": "1F6B8",
    "mens": "1F6B9",
    "womens": "1F6BA",
    "restroom": "1F6BB",
    "baby_symbol": "1F6BC",
    "toilet": "1F6BD",
    "wc": "1F6BE",
    "shower": "1F6BF",
    "bath": "1F6C0",
    "bathtub": "1F6C1",
    "passport_control": "1F6C2",
    "customs": "1F6C3",
    "baggage_claim": "1F6C4",
    "left_luggage": "1F6C5",
    "couch_and_lamp": "1F6CB-FE0F",
    "sleeping_accommodation": "1F6CC",
    "shopping_bags": "1F6CD-FE0F",
    "bellhop_bell": "1F6CE-FE0F",
    "bed": "1F6CF-FE0F",
    "place_of_worship": "1F6D0",
    "octagonal_sign": "1F6D1",
    "shopping_trolley": "1F6D2",
    "hindu_temple": "1F6D5",
    "hut": "1F6D6",
    "elevator": "1F6D7",
    "hammer_and_wrench": "1F6E0-FE0F",
    "shield": "1F6E1-FE0F",
    "oil_drum": "1F6E2-FE0F",
    "motorway": "1F6E3-FE0F",
    "railway_track": "1F6E4-FE0F",
    "motor_boat": "1F6E5-FE0F",
    "small_airplane": "1F6E9-FE0F",
    "airplane_departure": "1F6EB",
    "airplane_arriving": "1F6EC",
    "satellite": "1F6F0-FE0F",
    "passenger_ship": "1F6F3-FE0F",
    "scooter": "1F6F4",
    "motor_scooter": "1F6F5",
    "canoe": "1F6F6",
    "sled": "1F6F7",
    "flying_saucer": "1F6F8",
    "skateboard": "1F6F9",
    "auto_rickshaw": "1F6FA",
    "pickup_truck": "1F6FB",
    "roller_skate": "1F6FC",
    "large_orange_circle": "1F7E0",
    "large_yellow_circle": "1F7E1",
    "large_green_circle": "1F7E2",
    "large_purple_circle": "1F7E3",
    "large_brown_circle": "1F7E4",
    "large_red_square": "1F7E5",
    "large_blue_square": "1F7E6",
    "large_orange_square": "1F7E7",
    "large_yellow_square": "1F7E8",
    "large_green_square": "1F7E9",
    "large_purple_square": "1F7EA",
    "large_brown_square": "1F7EB",
    "pinched_fingers": "1F90C",
    "white_heart": "1F90D",
    "brown_heart": "1F90E",
    "pinching_hand": "1F90F",
    "zipper_mouth_face": "1F910",
    "money_mouth_face": "1F911",
    "face_with_thermometer": "1F912",
    "nerd_face": "1F913",
    "thinking_face": "1F914",
    "face_with_head_bandage": "1F915",
    "robot_face": "1F916",
    "hugging_face": "1F917",
    "the_horns": "1F918",
    "sign_of_the_horns": "1F918",
    "call_me_hand": "1F919",
    "raised_back_of_hand": "1F91A",
    "left-facing_fist": "1F91B",
    "right-facing_fist": "1F91C",
    "handshake": "1F91D",
    "crossed_fingers": "1F91E",
    "hand_with_index_and_middle_fingers_crossed": "1F91E",
    "i_love_you_hand_sign": "1F91F",
    "face_with_cowboy_hat": "1F920",
    "clown_face": "1F921",
    "nauseated_face": "1F922",
    "rolling_on_the_floor_laughing": "1F923",
    "drooling_face": "1F924",
    "lying_face": "1F925",
    "woman-facepalming": "1F926-200D-2640-FE0F",
    "man-facepalming": "1F926-200D-2642-FE0F",
    "face_palm": "1F926",
    "sneezing_face": "1F927",
    "face_with_raised_eyebrow": "1F928",
    "face_with_one_eyebrow_raised": "1F928",
    "star-struck": "1F929",
    "grinning_face_with_star_eyes": "1F929",
    "zany_face": "1F92A",
    "grinning_face_with_one_large_and_one_small_eye": "1F92A",
    "shushing_face": "1F92B",
    "face_with_finger_covering_closed_lips": "1F92B",
    "face_with_symbols_on_mouth": "1F92C",
    "serious_face_with_symbols_covering_mouth": "1F92C",
    "face_with_hand_over_mouth": "1F92D",
    "smiling_face_with_smiling_eyes_and_hand_covering_mouth": "1F92D",
    "face_vomiting": "1F92E",
    "face_with_open_mouth_vomiting": "1F92E",
    "exploding_head": "1F92F",
    "shocked_face_with_exploding_head": "1F92F",
    "pregnant_woman": "1F930",
    "breast-feeding": "1F931",
    "palms_up_together": "1F932",
    "selfie": "1F933",
    "prince": "1F934",
    "woman_in_tuxedo": "1F935-200D-2640-FE0F",
    "man_in_tuxedo": "1F935",
    "mrs_claus": "1F936",
    "mother_christmas": "1F936",
    "woman-shrugging": "1F937-200D-2640-FE0F",
    "man-shrugging": "1F937-200D-2642-FE0F",
    "shrug": "1F937",
    "woman-cartwheeling": "1F938-200D-2640-FE0F",
    "man-cartwheeling": "1F938-200D-2642-FE0F",
    "person_doing_cartwheel": "1F938",
    "woman-juggling": "1F939-200D-2640-FE0F",
    "man-juggling": "1F939-200D-2642-FE0F",
    "juggling": "1F939",
    "fencer": "1F93A",
    "woman-wrestling": "1F93C-200D-2640-FE0F",
    "man-wrestling": "1F93C-200D-2642-FE0F",
    "wrestlers": "1F93C",
    "woman-playing-water-polo": "1F93D-200D-2640-FE0F",
    "man-playing-water-polo": "1F93D-200D-2642-FE0F",
    "water_polo": "1F93D",
    "woman-playing-handball": "1F93E-200D-2640-FE0F",
    "man-playing-handball": "1F93E-200D-2642-FE0F",
    "handball": "1F93E",
    "diving_mask": "1F93F",
    "wilted_flower": "1F940",
    "drum_with_drumsticks": "1F941",
    "clinking_glasses": "1F942",
    "tumbler_glass": "1F943",
    "spoon": "1F944",
    "goal_net": "1F945",
    "first_place_medal": "1F947",
    "second_place_medal": "1F948",
    "third_place_medal": "1F949",
    "boxing_glove": "1F94A",
    "martial_arts_uniform": "1F94B",
    "curling_stone": "1F94C",
    "lacrosse": "1F94D",
    "softball": "1F94E",
    "flying_disc": "1F94F",
    "croissant": "1F950",
    "avocado": "1F951",
    "cucumber": "1F952",
    "bacon": "1F953",
    "potato": "1F954",
    "carrot": "1F955",
    "baguette_bread": "1F956",
    "green_salad": "1F957",
    "shallow_pan_of_food": "1F958",
    "stuffed_flatbread": "1F959",
    "egg": "1F95A",
    "glass_of_milk": "1F95B",
    "peanuts": "1F95C",
    "kiwifruit": "1F95D",
    "pancakes": "1F95E",
    "dumpling": "1F95F",
    "fortune_cookie": "1F960",
    "takeout_box": "1F961",
    "chopsticks": "1F962",
    "bowl_with_spoon": "1F963",
    "cup_with_straw": "1F964",
    "coconut": "1F965",
    "broccoli": "1F966",
    "pie": "1F967",
    "pretzel": "1F968",
    "cut_of_meat": "1F969",
    "sandwich": "1F96A",
    "canned_food": "1F96B",
    "leafy_green": "1F96C",
    "mango": "1F96D",
    "moon_cake": "1F96E",
    "bagel": "1F96F",
    "smiling_face_with_3_hearts": "1F970",
    "yawning_face": "1F971",
    "smiling_face_with_tear": "1F972",
    "partying_face": "1F973",
    "woozy_face": "1F974",
    "hot_face": "1F975",
    "cold_face": "1F976",
    "ninja": "1F977",
    "disguised_face": "1F978",
    "pleading_face": "1F97A",
    "sari": "1F97B",
    "lab_coat": "1F97C",
    "goggles": "1F97D",
    "hiking_boot": "1F97E",
    "womans_flat_shoe": "1F97F",
    "crab": "1F980",
    "lion_face": "1F981",
    "scorpion": "1F982",
    "turkey": "1F983",
    "unicorn_face": "1F984",
    "eagle": "1F985",
    "duck": "1F986",
    "bat": "1F987",
    "shark": "1F988",
    "owl": "1F989",
    "fox_face": "1F98A",
    "butterfly": "1F98B",
    "deer": "1F98C",
    "gorilla": "1F98D",
    "lizard": "1F98E",
    "rhinoceros": "1F98F",
    "shrimp": "1F990",
    "squid": "1F991",
    "giraffe_face": "1F992",
    "zebra_face": "1F993",
    "hedgehog": "1F994",
    "sauropod": "1F995",
    "t-rex": "1F996",
    "cricket": "1F997",
    "kangaroo": "1F998",
    "llama": "1F999",
    "peacock": "1F99A",
    "hippopotamus": "1F99B",
    "parrot": "1F99C",
    "raccoon": "1F99D",
    "lobster": "1F99E",
    "mosquito": "1F99F",
    "microbe": "1F9A0",
    "badger": "1F9A1",
    "swan": "1F9A2",
    "mammoth": "1F9A3",
    "dodo": "1F9A4",
    "sloth": "1F9A5",
    "otter": "1F9A6",
    "orangutan": "1F9A7",
    "skunk": "1F9A8",
    "flamingo": "1F9A9",
    "oyster": "1F9AA",
    "beaver": "1F9AB",
    "bison": "1F9AC",
    "seal": "1F9AD",
    "guide_dog": "1F9AE",
    "probing_cane": "1F9AF",
    "bone": "1F9B4",
    "leg": "1F9B5",
    "foot": "1F9B6",
    "tooth": "1F9B7",
    "female_superhero": "1F9B8-200D-2640-FE0F",
    "male_superhero": "1F9B8-200D-2642-FE0F",
    "superhero": "1F9B8",
    "female_supervillain": "1F9B9-200D-2640-FE0F",
    "male_supervillain": "1F9B9-200D-2642-FE0F",
    "supervillain": "1F9B9",
    "safety_vest": "1F9BA",
    "ear_with_hearing_aid": "1F9BB",
    "motorized_wheelchair": "1F9BC",
    "manual_wheelchair": "1F9BD",
    "mechanical_arm": "1F9BE",
    "mechanical_leg": "1F9BF",
    "cheese_wedge": "1F9C0",
    "cupcake": "1F9C1",
    "salt": "1F9C2",
    "beverage_box": "1F9C3",
    "garlic": "1F9C4",
    "onion": "1F9C5",
    "falafel": "1F9C6",
    "waffle": "1F9C7",
    "butter": "1F9C8",
    "mate_drink": "1F9C9",
    "ice_cube": "1F9CA",
    "bubble_tea": "1F9CB",
    "woman_standing": "1F9CD-200D-2640-FE0F",
    "man_standing": "1F9CD-200D-2642-FE0F",
    "standing_person": "1F9CD",
    "woman_kneeling": "1F9CE-200D-2640-FE0F",
    "man_kneeling": "1F9CE-200D-2642-FE0F",
    "kneeling_person": "1F9CE",
    "deaf_woman": "1F9CF-200D-2640-FE0F",
    "deaf_man": "1F9CF-200D-2642-FE0F",
    "deaf_person": "1F9CF",
    "face_with_monocle": "1F9D0",
    "farmer": "1F9D1-200D-1F33E",
    "cook": "1F9D1-200D-1F373",
    "person_feeding_baby": "1F9D1-200D-1F37C",
    "mx_claus": "1F9D1-200D-1F384",
    "student": "1F9D1-200D-1F393",
    "singer": "1F9D1-200D-1F3A4",
    "artist": "1F9D1-200D-1F3A8",
    "teacher": "1F9D1-200D-1F3EB",
    "factory_worker": "1F9D1-200D-1F3ED",
    "technologist": "1F9D1-200D-1F4BB",
    "office_worker": "1F9D1-200D-1F4BC",
    "mechanic": "1F9D1-200D-1F527",
    "scientist": "1F9D1-200D-1F52C",
    "astronaut": "1F9D1-200D-1F680",
    "firefighter": "1F9D1-200D-1F692",
    "people_holding_hands": "1F9D1-200D-1F91D-200D-1F9D1",
    "person_with_probing_cane": "1F9D1-200D-1F9AF",
    "red_haired_person": "1F9D1-200D-1F9B0",
    "curly_haired_person": "1F9D1-200D-1F9B1",
    "bald_person": "1F9D1-200D-1F9B2",
    "white_haired_person": "1F9D1-200D-1F9B3",
    "person_in_motorized_wheelchair": "1F9D1-200D-1F9BC",
    "person_in_manual_wheelchair": "1F9D1-200D-1F9BD",
    "health_worker": "1F9D1-200D-2695-FE0F",
    "judge": "1F9D1-200D-2696-FE0F",
    "pilot": "1F9D1-200D-2708-FE0F",
    "adult": "1F9D1",
    "child": "1F9D2",
    "older_adult": "1F9D3",
    "bearded_person": "1F9D4",
    "person_with_headscarf": "1F9D5",
    "woman_in_steamy_room": "1F9D6-200D-2640-FE0F",
    "man_in_steamy_room": "1F9D6-200D-2642-FE0F",
    "person_in_steamy_room": "1F9D6",
    "woman_climbing": "1F9D7-200D-2640-FE0F",
    "man_climbing": "1F9D7-200D-2642-FE0F",
    "person_climbing": "1F9D7",
    "woman_in_lotus_position": "1F9D8-200D-2640-FE0F",
    "man_in_lotus_position": "1F9D8-200D-2642-FE0F",
    "person_in_lotus_position": "1F9D8",
    "female_mage": "1F9D9-200D-2640-FE0F",
    "male_mage": "1F9D9-200D-2642-FE0F",
    "mage": "1F9D9",
    "female_fairy": "1F9DA-200D-2640-FE0F",
    "male_fairy": "1F9DA-200D-2642-FE0F",
    "fairy": "1F9DA",
    "female_vampire": "1F9DB-200D-2640-FE0F",
    "male_vampire": "1F9DB-200D-2642-FE0F",
    "vampire": "1F9DB",
    "mermaid": "1F9DC-200D-2640-FE0F",
    "merman": "1F9DC-200D-2642-FE0F",
    "merperson": "1F9DC",
    "female_elf": "1F9DD-200D-2640-FE0F",
    "male_elf": "1F9DD-200D-2642-FE0F",
    "elf": "1F9DD",
    "female_genie": "1F9DE-200D-2640-FE0F",
    "male_genie": "1F9DE-200D-2642-FE0F",
    "genie": "1F9DE",
    "female_zombie": "1F9DF-200D-2640-FE0F",
    "male_zombie": "1F9DF-200D-2642-FE0F",
    "zombie": "1F9DF",
    "brain": "1F9E0",
    "orange_heart": "1F9E1",
    "billed_cap": "1F9E2",
    "scarf": "1F9E3",
    "gloves": "1F9E4",
    "coat": "1F9E5",
    "socks": "1F9E6",
    "red_envelope": "1F9E7",
    "firecracker": "1F9E8",
    "jigsaw": "1F9E9",
    "test_tube": "1F9EA",
    "petri_dish": "1F9EB",
    "dna": "1F9EC",
    "compass": "1F9ED",
    "abacus": "1F9EE",
    "fire_extinguisher": "1F9EF",
    "toolbox": "1F9F0",
    "bricks": "1F9F1",
    "magnet": "1F9F2",
    "luggage": "1F9F3",
    "lotion_bottle": "1F9F4",
    "thread": "1F9F5",
    "yarn": "1F9F6",
    "safety_pin": "1F9F7",
    "teddy_bear": "1F9F8",
    "broom": "1F9F9",
    "basket": "1F9FA",
    "roll_of_paper": "1F9FB",
    "soap": "1F9FC",
    "sponge": "1F9FD",
    "receipt": "1F9FE",
    "nazar_amulet": "1F9FF",
    "ballet_shoes": "1FA70",
    "one-piece_swimsuit": "1FA71",
    "briefs": "1FA72",
    "shorts": "1FA73",
    "thong_sandal": "1FA74",
    "drop_of_blood": "1FA78",
    "adhesive_bandage": "1FA79",
    "stethoscope": "1FA7A",
    "yo-yo": "1FA80",
    "kite": "1FA81",
    "parachute": "1FA82",
    "boomerang": "1FA83",
    "magic_wand": "1FA84",
    "pinata": "1FA85",
    "nesting_dolls": "1FA86",
    "ringed_planet": "1FA90",
    "chair": "1FA91",
    "razor": "1FA92",
    "axe": "1FA93",
    "diya_lamp": "1FA94",
    "banjo": "1FA95",
    "military_helmet": "1FA96",
    "accordion": "1FA97",
    "long_drum": "1FA98",
    "coin": "1FA99",
    "carpentry_saw": "1FA9A",
    "screwdriver": "1FA9B",
    "ladder": "1FA9C",
    "hook": "1FA9D",
    "mirror": "1FA9E",
    "window": "1FA9F",
    "plunger": "1FAA0",
    "sewing_needle": "1FAA1",
    "knot": "1FAA2",
    "bucket": "1FAA3",
    "mouse_trap": "1FAA4",
    "toothbrush": "1FAA5",
    "headstone": "1FAA6",
    "placard": "1FAA7",
    "rock": "1FAA8",
    "fly": "1FAB0",
    "worm": "1FAB1",
    "cockroach": "1FAB3",
    "potted_plant": "1FAB4",
    "wood": "1FAB5",
    "feather": "1FAB6",
    "anatomical_heart": "1FAC0",
    "lungs": "1FAC1",
    "people_hugging": "1FAC2",
    "blueberries": "1FAD0",
    "bell_pepper": "1FAD1",
    "olive": "1FAD2",
    "flatbread": "1FAD3",
    "tamale": "1FAD4",
    "fondue": "1FAD5",
    "teapot": "1FAD6",
    "bangbang": "203C-FE0F",
    "interrobang": "2049-FE0F",
    "tm": "2122-FE0F",
    "information_source": "2139-FE0F",
    "left_right_arrow": "2194-FE0F",
    "arrow_up_down": "2195-FE0F",
    "arrow_upper_left": "2196-FE0F",
    "arrow_upper_right": "2197-FE0F",
    "arrow_lower_right": "2198-FE0F",
    "arrow_lower_left": "2199-FE0F",
    "leftwards_arrow_with_hook": "21A9-FE0F",
    "arrow_right_hook": "21AA-FE0F",
    "watch": "231A",
    "hourglass": "231B",
    "keyboard": "2328-FE0F",
    "eject": "23CF-FE0F",
    "fast_forward": "23E9",
    "rewind": "23EA",
    "arrow_double_up": "23EB",
    "arrow_double_down": "23EC",
    "black_right_pointing_double_triangle_with_vertical_bar": "23ED-FE0F",
    "black_left_pointing_double_triangle_with_vertical_bar": "23EE-FE0F",
    "black_right_pointing_triangle_with_double_vertical_bar": "23EF-FE0F",
    "alarm_clock": "23F0",
    "stopwatch": "23F1-FE0F",
    "timer_clock": "23F2-FE0F",
    "hourglass_flowing_sand": "23F3",
    "double_vertical_bar": "23F8-FE0F",
    "black_square_for_stop": "23F9-FE0F",
    "black_circle_for_record": "23FA-FE0F",
    "m": "24C2-FE0F",
    "black_small_square": "25AA-FE0F",
    "white_small_square": "25AB-FE0F",
    "arrow_forward": "25B6-FE0F",
    "arrow_backward": "25C0-FE0F",
    "white_medium_square": "25FB-FE0F",
    "black_medium_square": "25FC-FE0F",
    "white_medium_small_square": "25FD",
    "black_medium_small_square": "25FE",
    "sunny": "2600-FE0F",
    "cloud": "2601-FE0F",
    "umbrella": "2602-FE0F",
    "snowman": "2603-FE0F",
    "comet": "2604-FE0F",
    "phone": "260E-FE0F",
    "telephone": "260E-FE0F",
    "ballot_box_with_check": "2611-FE0F",
    "umbrella_with_rain_drops": "2614",
    "coffee": "2615",
    "shamrock": "2618-FE0F",
    "point_up": "261D-FE0F",
    "skull_and_crossbones": "2620-FE0F",
    "radioactive_sign": "2622-FE0F",
    "biohazard_sign": "2623-FE0F",
    "orthodox_cross": "2626-FE0F",
    "star_and_crescent": "262A-FE0F",
    "peace_symbol": "262E-FE0F",
    "yin_yang": "262F-FE0F",
    "wheel_of_dharma": "2638-FE0F",
    "white_frowning_face": "2639-FE0F",
    "relaxed": "263A-FE0F",
    "female_sign": "2640-FE0F",
    "male_sign": "2642-FE0F",
    "aries": "2648",
    "taurus": "2649",
    "gemini": "264A",
    "cancer": "264B",
    "leo": "264C",
    "virgo": "264D",
    "libra": "264E",
    "scorpius": "264F",
    "sagittarius": "2650",
    "capricorn": "2651",
    "aquarius": "2652",
    "pisces": "2653",
    "chess_pawn": "265F-FE0F",
    "spades": "2660-FE0F",
    "clubs": "2663-FE0F",
    "hearts": "2665-FE0F",
    "diamonds": "2666-FE0F",
    "hotsprings": "2668-FE0F",
    "recycle": "267B-FE0F",
    "infinity": "267E-FE0F",
    "wheelchair": "267F",
    "hammer_and_pick": "2692-FE0F",
    "anchor": "2693",
    "crossed_swords": "2694-FE0F",
    "medical_symbol": "2695-FE0F",
    "staff_of_aesculapius": "2695-FE0F",
    "scales": "2696-FE0F",
    "alembic": "2697-FE0F",
    "gear": "2699-FE0F",
    "atom_symbol": "269B-FE0F",
    "fleur_de_lis": "269C-FE0F",
    "warning": "26A0-FE0F",
    "zap": "26A1",
    "transgender_symbol": "26A7-FE0F",
    "white_circle": "26AA",
    "black_circle": "26AB",
    "coffin": "26B0-FE0F",
    "funeral_urn": "26B1-FE0F",
    "soccer": "26BD",
    "baseball": "26BE",
    "snowman_without_snow": "26C4",
    "partly_sunny": "26C5",
    "thunder_cloud_and_rain": "26C8-FE0F",
    "ophiuchus": "26CE",
    "pick": "26CF-FE0F",
    "helmet_with_white_cross": "26D1-FE0F",
    "chains": "26D3-FE0F",
    "no_entry": "26D4",
    "shinto_shrine": "26E9-FE0F",
    "church": "26EA",
    "mountain": "26F0-FE0F",
    "umbrella_on_ground": "26F1-FE0F",
    "fountain": "26F2",
    "golf": "26F3",
    "ferry": "26F4-FE0F",
    "boat": "26F5",
    "sailboat": "26F5",
    "skier": "26F7-FE0F",
    "ice_skate": "26F8-FE0F",
    "woman-bouncing-ball": "26F9-FE0F-200D-2640-FE0F",
    "man-bouncing-ball": "26F9-FE0F-200D-2642-FE0F",
    "person_with_ball": "26F9-FE0F",
    "tent": "26FA",
    "fuelpump": "26FD",
    "scissors": "2702-FE0F",
    "white_check_mark": "2705",
    "airplane": "2708-FE0F",
    "email": "2709-FE0F",
    "envelope": "2709-FE0F",
    "fist": "270A",
    "hand": "270B",
    "raised_hand": "270B",
    "v": "270C-FE0F",
    "writing_hand": "270D-FE0F",
    "pencil2": "270F-FE0F",
    "black_nib": "2712-FE0F",
    "heavy_check_mark": "2714-FE0F",
    "heavy_multiplication_x": "2716-FE0F",
    "latin_cross": "271D-FE0F",
    "star_of_david": "2721-FE0F",
    "sparkles": "2728",
    "eight_spoked_asterisk": "2733-FE0F",
    "eight_pointed_black_star": "2734-FE0F",
    "snowflake": "2744-FE0F",
    "sparkle": "2747-FE0F",
    "x": "274C",
    "negative_squared_cross_mark": "274E",
    "question": "2753",
    "grey_question": "2754",
    "grey_exclamation": "2755",
    "exclamation": "2757",
    "heavy_exclamation_mark": "2757",
    "heavy_heart_exclamation_mark_ornament": "2763-FE0F",
    "heart": "2764-FE0F",
    "heavy_plus_sign": "2795",
    "heavy_minus_sign": "2796",
    "heavy_division_sign": "2797",
    "arrow_right": "27A1-FE0F",
    "curly_loop": "27B0",
    "loop": "27BF",
    "arrow_heading_up": "2934-FE0F",
    "arrow_heading_down": "2935-FE0F",
    "arrow_left": "2B05-FE0F",
    "arrow_up": "2B06-FE0F",
    "arrow_down": "2B07-FE0F",
    "black_large_square": "2B1B",
    "white_large_square": "2B1C",
    "star": "2B50",
    "o": "2B55",
    "wavy_dash": "3030-FE0F",
    "part_alternation_mark": "303D-FE0F",
    "congratulations": "3297-FE0F",
    "secret": "3299-FE0F",
    "person_getting_haircut_tone1": "1f487-1f3fb",
    "woman_elf_tone4": "1f9dd-1f3fe-2640",
    "man_in_steamy_room_tone4": "1f9d6-1f3fe-2642",
    "man_playing_water_polo_tone3": "1f93d-1f3fd-2642",
    "flag_by": "1f1e7-1f1fe",
    "flag_lc": "1f1f1-1f1e8",
    "flag_mf": "1f1f2-1f1eb",
    "older_man_tone1": "1f474-1f3fb",
    "desktop": "1f5a5",
    "person_surfing": "1f3c4",
    "beach_umbrella": "26f1",
    "mrs_claus_tone5": "1f936-1f3ff",
    "raised_hands_tone3": "1f64c-1f3fd",
    "person_fencing": "1f93a",
    "man_technologist_tone2": "1f468-1f3fc-1f4bb",
    "man_cook_tone4": "1f468-1f3fe-1f373",
    "open_hands_tone4": "1f450-1f3fe",
    "man_climbing_tone2": "1f9d7-1f3fc-2642",
    "dove": "1f54a",
    "person_bowing_tone5": "1f647-1f3ff",
    "metal_tone5": "1f918-1f3ff",
    "vampire_tone4": "1f9db-1f3fe",
    "regional_indicator_e": "1f1ea",
    "flag_pe": "1f1f5-1f1ea",
    "mrs_claus_tone1": "1f936-1f3fb",
    "man_mechanic": "1f468-1f527",
    "woman_lifting_weights": "1f3cb-2640",
    "person_gesturing_no_tone3": "1f645-1f3fd",
    "man_swimming_tone3": "1f3ca-1f3fd-2642",
    "man_walking_tone4": "1f6b6-1f3fe-2642",
    "man_police_officer_tone3": "1f46e-1f3fd-2642",
    "kiss_mm": "1f468-2764-1f48b-1f468",
    "man_mage_tone3": "1f9d9-1f3fd-2642",
    "raised_back_of_hand_tone4": "1f91a-1f3fe",
    "fox": "1f98a",
    "flag_mg": "1f1f2-1f1ec",
    "boy_tone4": "1f466-1f3fe",
    "flag_ro": "1f1f7-1f1f4",
    "women_wrestling": "1f93c-2640",
    "map": "1f5fa",
    "right_facing_fist": "1f91c",
    "man_getting_face_massage_tone4": "1f486-1f3fe-2642",
    "woman_lifting_weights_tone4": "1f3cb-1f3fe-2640",
    "bald": "1f9b2",
    "woman_biking_tone2": "1f6b4-1f3fc-2640",
    "man_fairy_tone2": "1f9da-1f3fc-2642",
    "person_running_tone4": "1f3c3-1f3fe",
    "wales": "1f3f4-e0067-e0062-e0077-e006c-e0073-e007f",
    "man_tipping_hand_tone4": "1f481-1f3fe-2642",
    "woman_biking_tone3": "1f6b4-1f3fd-2640",
    "detective": "1f575",
    "flag_mz": "1f1f2-1f1ff",
    "foot_tone3": "1f9b6-1f3fd",
    "man_factory_worker_tone2": "1f468-1f3fc-1f3ed",
    "mermaid_tone5": "1f9dc-1f3ff-2640",
    "palms_up_together_tone5": "1f932-1f3ff",
    "vulcan_tone1": "1f596-1f3fb",
    "flag_aq": "1f1e6-1f1f6",
    "woman_firefighter_tone3": "1f469-1f3fd-1f692",
    "flag_ee": "1f1ea-1f1ea",
    "mage_tone5": "1f9d9-1f3ff",
    "man_running_tone5": "1f3c3-1f3ff-2642",
    "woman_walking": "1f6b6-2640",
    "levitate_tone4": "1f574-1f3fe",
    "play_pause": "23ef",
    "person_mountain_biking_tone5": "1f6b5-1f3ff",
    "woman_mechanic_tone2": "1f469-1f3fc-1f527",
    "person_in_bed_tone4": "1f6cc-1f3fe",
    "woman_farmer_tone4": "1f469-1f3fe-1f33e",
    "flag_cm": "1f1e8-1f1f2",
    "flag_bg": "1f1e7-1f1ec",
    "person_bouncing_ball_tone1": "26f9-1f3fb",
    "flag_me": "1f1f2-1f1ea",
    "woman_detective_tone2": "1f575-1f3fc-2640",
    "person_juggling_tone5": "1f939-1f3ff",
    "city_dusk": "1f306",
    "child_tone4": "1f9d2-1f3fe",
    "person_gesturing_no_tone1": "1f645-1f3fb",
    "person_playing_water_polo_tone5": "1f93d-1f3ff",
    "regional_indicator_i": "1f1ee",
    "woman_white_haired_tone1": "1f469-1f3fb-1f9b3",
    "flag_lb": "1f1f1-1f1e7",
    "princess_tone3": "1f478-1f3fd",
    "man_elf_tone4": "1f9dd-1f3fe-2642",
    "bearded_person_tone4": "1f9d4-1f3fe",
    "fingers_crossed_tone3": "1f91e-1f3fd",
    "woman_facepalming_tone1": "1f926-1f3fb-2640",
    "woman_frowning": "1f64d-2640",
    "woman_juggling": "1f939-2640",
    "writing_hand_tone3": "270d-1f3fd",
    "construction_worker_tone3": "1f477-1f3fd",
    "woman_curly_haired": "1f469-1f9b1",
    "woman_lifting_weights_tone5": "1f3cb-1f3ff-2640",
    "point_up_tone4": "261d-1f3fe",
    "writing_hand_tone5": "270d-1f3ff",
    "regional_indicator_x": "1f1fd",
    "helmet_with_cross": "26d1",
    "man_biking_tone1": "1f6b4-1f3fb-2642",
    "man_biking_tone4": "1f6b4-1f3fe-2642",
    "woman_pouting_tone3": "1f64e-1f3fd-2640",
    "elf_tone3": "1f9dd-1f3fd",
    "flag_black": "1f3f4",
    "v_tone5": "270c-1f3ff",
    "writing_hand_tone4": "270d-1f3fe",
    "vulcan_tone4": "1f596-1f3fe",
    "woman_cartwheeling_tone2": "1f938-1f3fc-2640",
    "flag_bs": "1f1e7-1f1f8",
    "construction_site": "1f3d7",
    "family_woman_boy": "1f469-1f466",
    "woman_playing_water_polo_tone1": "1f93d-1f3fb-2640",
    "man_lifting_weights_tone5": "1f3cb-1f3ff-2642",
    "flag_de": "1f1e9-1f1ea",
    "woman_artist_tone2": "1f469-1f3fc-1f3a8",
    "elf_tone1": "1f9dd-1f3fb",
    "clap_tone4": "1f44f-1f3fe",
    "woman_mechanic_tone1": "1f469-1f3fb-1f527",
    "regional_indicator_y": "1f1fe",
    "fingers_crossed_tone5": "1f91e-1f3ff",
    "man_judge": "1f468-2696",
    "white_haired": "1f9b3",
    "person_in_steamy_room_tone2": "1f9d6-1f3fc",
    "woman_judge": "1f469-2696",
    "family_wwg": "1f469-1f469-1f467",
    "man_health_worker_tone5": "1f468-1f3ff-2695",
    "man_student_tone1": "1f468-1f3fb-1f393",
    "person_juggling_tone1": "1f939-1f3fb",
    "woman_astronaut_tone1": "1f469-1f3fb-1f680",
    "stop_button": "23f9",
    "man_health_worker_tone4": "1f468-1f3fe-2695",
    "woman_student_tone3": "1f469-1f3fd-1f393",
    "person_raising_hand_tone5": "1f64b-1f3ff",
    "vampire_tone1": "1f9db-1f3fb",
    "man_surfing_tone3": "1f3c4-1f3fd-2642",
    "woman_surfing_tone1": "1f3c4-1f3fb-2640",
    "ear_tone1": "1f442-1f3fb",
    "heart_exclamation": "2763",
    "goal": "1f945",
    "woman_gesturing_ok_tone3": "1f646-1f3fd-2640",
    "woman_tipping_hand_tone5": "1f481-1f3ff-2640",
    "love_you_gesture_tone2": "1f91f-1f3fc",
    "metal_tone2": "1f918-1f3fc",
    "person_in_lotus_position_tone5": "1f9d8-1f3ff",
    "zebra": "1f993",
    "england": "1f3f4-e0067-e0062-e0065-e006e-e0067-e007f",
    "man_getting_haircut_tone4": "1f487-1f3fe-2642",
    "man_bouncing_ball": "26f9-2642",
    "pause_button": "23f8",
    "woman_lifting_weights_tone1": "1f3cb-1f3fb-2640",
    "man_golfing": "1f3cc-2642",
    "person_swimming_tone1": "1f3ca-1f3fb",
    "flag_gf": "1f1ec-1f1eb",
    "woman_in_lotus_position_tone4": "1f9d8-1f3fe-2640",
    "woman_astronaut_tone2": "1f469-1f3fc-1f680",
    "flag_th": "1f1f9-1f1ed",
    "flag_sc": "1f1f8-1f1e8",
    "flag_gg": "1f1ec-1f1ec",
    "airplane_small": "1f6e9",
    "princess_tone5": "1f478-1f3ff",
    "woman_teacher_tone5": "1f469-1f3ff-1f3eb",
    "flag_fr": "1f1eb-1f1f7",
    "person_pouting_tone3": "1f64e-1f3fd",
    "flag_us": "1f1fa-1f1f8",
    "angel_tone5": "1f47c-1f3ff",
    "open_hands_tone2": "1f450-1f3fc",
    "child_tone3": "1f9d2-1f3fd",
    "woman_supervillain_tone2": "1f9b9-1f3fc-2640",
    "flag_ac": "1f1e6-1f1e8",
    "man_swimming_tone5": "1f3ca-1f3ff-2642",
    "flag_pt": "1f1f5-1f1f9",
    "woman_police_officer_tone2": "1f46e-1f3fc-2640",
    "urn": "26b1",
    "man_judge_tone4": "1f468-1f3fe-2696",
    "notepad_spiral": "1f5d2",
    "breast_feeding_tone5": "1f931-1f3ff",
    "man_playing_handball_tone5": "1f93e-1f3ff-2642",
    "superhero_tone2": "1f9b8-1f3fc",
    "flag_jo": "1f1ef-1f1f4",
    "left_facing_fist_tone3": "1f91b-1f3fd",
    "man_juggling_tone1": "1f939-1f3fb-2642",
    "baby_tone4": "1f476-1f3fe",
    "person_in_bed_tone2": "1f6cc-1f3fc",
    "man_getting_face_massage_tone3": "1f486-1f3fd-2642",
    "guard_tone5": "1f482-1f3ff",
    "man_red_haired_tone1": "1f468-1f3fb-1f9b0",
    "woman_singer": "1f469-1f3a4",
    "man_judge_tone2": "1f468-1f3fc-2696",
    "woman_construction_worker_tone3": "1f477-1f3fd-2640",
    "woman_climbing_tone1": "1f9d7-1f3fb-2640",
    "point_down_tone4": "1f447-1f3fe",
    "man_bouncing_ball_tone3": "26f9-1f3fd-2642",
    "woman_getting_haircut": "1f487-2640",
    "guard_tone3": "1f482-1f3fd",
    "red_haired": "1f9b0",
    "cloud_snow": "1f328",
    "flag_kp": "1f1f0-1f1f5",
    "angel_tone1": "1f47c-1f3fb",
    "flag_hk": "1f1ed-1f1f0",
    "man_mechanic_tone5": "1f468-1f3ff-1f527",
    "man_office_worker_tone2": "1f468-1f3fc-1f4bc",
    "woman_getting_face_massage_tone1": "1f486-1f3fb-2640",
    "bride_with_veil_tone1": "1f470-1f3fb",
    "pound_symbol": "0023",
    "person_playing_water_polo_tone2": "1f93d-1f3fc",
    "blond_haired_person": "1f471",
    "woman_getting_haircut_tone2": "1f487-1f3fc-2640",
    "flag_es": "1f1ea-1f1f8",
    "man_with_chinese_cap_tone4": "1f472-1f3fe",
    "woman_detective_tone1": "1f575-1f3fb-2640",
    "woman_superhero_tone1": "1f9b8-1f3fb-2640",
    "pray_tone3": "1f64f-1f3fd",
    "flag_co": "1f1e8-1f1f4",
    "ear_tone3": "1f442-1f3fd",
    "flag_cx": "1f1e8-1f1fd",
    "superhero_tone3": "1f9b8-1f3fd",
    "mermaid_tone4": "1f9dc-1f3fe-2640",
    "person_getting_haircut": "1f487",
    "person_frowning_tone1": "1f64d-1f3fb",
    "santa_tone2": "1f385-1f3fc",
    "person_playing_water_polo_tone1": "1f93d-1f3fb",
    "man_golfing_tone4": "1f3cc-1f3fe-2642",
    "person_frowning_tone2": "1f64d-1f3fc",
    "man_walking_tone3": "1f6b6-1f3fd-2642",
    "woman_factory_worker_tone5": "1f469-1f3ff-1f3ed",
    "man_fairy_tone3": "1f9da-1f3fd-2642",
    "person_playing_handball": "1f93e",
    "woman_construction_worker_tone5": "1f477-1f3ff-2640",
    "man_bald_tone2": "1f468-1f3fc-1f9b2",
    "woman_teacher_tone2": "1f469-1f3fc-1f3eb",
    "wave_tone2": "1f44b-1f3fc",
    "family_man_girl_boy": "1f468-1f467-1f466",
    "woman_juggling_tone3": "1f939-1f3fd-2640",
    "vulcan": "1f596",
    "man_in_tuxedo_tone4": "1f935-1f3fe",
    "person_tipping_hand_tone5": "1f481-1f3ff",
    "man_dancing_tone2": "1f57a-1f3fc",
    "homes": "1f3d8",
    "woman_fairy": "1f9da-2640",
    "flag_ae": "1f1e6-1f1ea",
    "older_adult_tone5": "1f9d3-1f3ff",
    "hockey": "1f3d2",
    "detective_tone1": "1f575-1f3fb",
    "man_office_worker_tone4": "1f468-1f3fe-1f4bc",
    "point_right_tone1": "1f449-1f3fb",
    "palms_up_together_tone1": "1f932-1f3fb",
    "blond_haired_person_tone4": "1f471-1f3fe",
    "man_astronaut_tone2": "1f468-1f3fc-1f680",
    "person_getting_haircut_tone4": "1f487-1f3fe",
    "regional_indicator_c": "1f1e8",
    "prince_tone4": "1f934-1f3fe",
    "person_swimming_tone4": "1f3ca-1f3fe",
    "man_guard_tone5": "1f482-1f3ff-2642",
    "woman_red_haired_tone1": "1f469-1f3fb-1f9b0",
    "person_swimming_tone2": "1f3ca-1f3fc",
    "woman_white_haired_tone2": "1f469-1f3fc-1f9b3",
    "woman_bouncing_ball": "26f9-2640",
    "woman_student_tone4": "1f469-1f3fe-1f393",
    "woman_wearing_turban_tone3": "1f473-1f3fd-2640",
    "person_getting_massage": "1f486",
    "love_you_gesture_tone3": "1f91f-1f3fd",
    "woman_facepalming_tone5": "1f926-1f3ff-2640",
    "woman_elf_tone1": "1f9dd-1f3fb-2640",
    "man_white_haired_tone5": "1f468-1f3ff-1f9b3",
    "person_doing_cartwheel_tone4": "1f938-1f3fe",
    "woman_genie": "1f9de-2640",
    "woman_bald_tone5": "1f469-1f3ff-1f9b2",
    "woman_getting_face_massage_tone4": "1f486-1f3fe-2640",
    "person_tipping_hand_tone2": "1f481-1f3fc",
    "thumbsdown_tone4": "1f44e-1f3fe",
    "metal_tone1": "1f918-1f3fb",
    "woman_singer_tone3": "1f469-1f3fd-1f3a4",
    "flag_my": "1f1f2-1f1fe",
    "love_you_gesture_tone1": "1f91f-1f3fb",
    "family_wwbb": "1f469-1f469-1f466-1f466",
    "key2": "1f5dd",
    "man_juggling_tone5": "1f939-1f3ff-2642",
    "man_bouncing_ball_tone2": "26f9-1f3fc-2642",
    "man_climbing_tone4": "1f9d7-1f3fe-2642",
    "person_juggling": "1f939",
    "man_guard_tone4": "1f482-1f3fe-2642",
    "flag_st": "1f1f8-1f1f9",
    "man_judge_tone5": "1f468-1f3ff-2696",
    "merperson_tone3": "1f9dc-1f3fd",
    "person_golfing_tone1": "1f3cc-1f3fb",
    "guard_tone1": "1f482-1f3fb",
    "woman_student_tone2": "1f469-1f3fc-1f393",
    "bath_tone5": "1f6c0-1f3ff",
    "person_biking_tone5": "1f6b4-1f3ff",
    "slight_smile": "1f642",
    "man_climbing_tone1": "1f9d7-1f3fb-2642",
    "woman_teacher_tone3": "1f469-1f3fd-1f3eb",
    "regional_indicator_j": "1f1ef",
    "person_walking_tone3": "1f6b6-1f3fd",
    "microphone2": "1f399",
    "robot": "1f916",
    "tone1": "1f3fb",
    "man_cook": "1f468-1f373",
    "eye_in_speech_bubble": "1f441-1f5e8",
    "man_wearing_turban_tone2": "1f473-1f3fc-2642",
    "woman_bouncing_ball_tone2": "26f9-1f3fc-2640",
    "woman_cook_tone1": "1f469-1f3fb-1f373",
    "woman_factory_worker": "1f469-1f3ed",
    "selfie_tone3": "1f933-1f3fd",
    "tone2": "1f3fc",
    "flag_tl": "1f1f9-1f1f1",
    "woman_walking_tone1": "1f6b6-1f3fb-2640",
    "woman_raising_hand_tone3": "1f64b-1f3fd-2640",
    "boy_tone1": "1f466-1f3fb",
    "woman_mage_tone1": "1f9d9-1f3fb-2640",
    "woman_biking_tone1": "1f6b4-1f3fb-2640",
    "woman_detective_tone5": "1f575-1f3ff-2640",
    "woman_getting_haircut_tone3": "1f487-1f3fd-2640",
    "newspaper2": "1f5de",
    "man_health_worker_tone2": "1f468-1f3fc-2695",
    "baby_tone5": "1f476-1f3ff",
    "person_surfing_tone4": "1f3c4-1f3fe",
    "pregnant_woman_tone4": "1f930-1f3fe",
    "flag_ni": "1f1f3-1f1ee",
    "person_bouncing_ball_tone5": "26f9-1f3ff",
    "call_me": "1f919",
    "woman_getting_face_massage": "1f486-2640",
    "flag_gt": "1f1ec-1f1f9",
    "person_wearing_turban_tone5": "1f473-1f3ff",
    "flag_gh": "1f1ec-1f1ed",
    "woman_surfing_tone5": "1f3c4-1f3ff-2640",
    "older_woman_tone2": "1f475-1f3fc",
    "flag_mw": "1f1f2-1f1fc",
    "dividers": "1f5c2",
    "kiss_woman_man": "1f469-2764-1f48b-1f468",
    "police_officer": "1f46e",
    "family_mwgb": "1f468-1f469-1f467-1f466",
    "point_up_2_tone1": "1f446-1f3fb",
    "woman_running_tone2": "1f3c3-1f3fc-2640",
    "flag_md": "1f1f2-1f1e9",
    "woman_judge_tone1": "1f469-1f3fb-2696",
    "person_facepalming": "1f926",
    "people_wrestling": "1f93c",
    "man_wearing_turban": "1f473-2642",
    "bride_with_veil_tone4": "1f470-1f3fe",
    "flag_tr": "1f1f9-1f1f7",
    "person_in_bed_tone1": "1f6cc-1f3fb",
    "flag_ge": "1f1ec-1f1ea",
    "blond-haired_woman": "1f471-2640",
    "construction_worker_tone5": "1f477-1f3ff",
    "man_cartwheeling_tone5": "1f938-1f3ff-2642",
    "man_pilot_tone2": "1f468-1f3fc-2708",
    "flag_fm": "1f1eb-1f1f2",
    "woman_wearing_turban_tone1": "1f473-1f3fb-2640",
    "snowboarder_tone4": "1f3c2-1f3fe",
    "flag_er": "1f1ea-1f1f7",
    "older_woman_tone5": "1f475-1f3ff",
    "man_mage_tone5": "1f9d9-1f3ff-2642",
    "man_artist_tone2": "1f468-1f3fc-1f3a8",
    "man_wearing_turban_tone3": "1f473-1f3fd-2642",
    "horse_racing_tone4": "1f3c7-1f3fe",
    "punch_tone5": "1f44a-1f3ff",
    "man_bouncing_ball_tone5": "26f9-1f3ff-2642",
    "giraffe": "1f992",
    "supervillain_tone2": "1f9b9-1f3fc",
    "bearded_person_tone3": "1f9d4-1f3fd",
    "woman_technologist_tone1": "1f469-1f3fb-1f4bb",
    "woman_raising_hand_tone1": "1f64b-1f3fb-2640",
    "man_superhero_tone1": "1f9b8-1f3fb-2642",
    "point_down_tone3": "1f447-1f3fd",
    "person_surfing_tone2": "1f3c4-1f3fc",
    "baby_tone2": "1f476-1f3fc",
    "woman_bouncing_ball_tone1": "26f9-1f3fb-2640",
    "princess_tone4": "1f478-1f3fe",
    "raised_hand_tone4": "270b-1f3fe",
    "blond-haired_man_tone3": "1f471-1f3fd-2642",
    "woman_farmer": "1f469-1f33e",
    "woman_judge_tone2": "1f469-1f3fc-2696",
    "woman_mage_tone2": "1f9d9-1f3fc-2640",
    "flag_gw": "1f1ec-1f1fc",
    "person_facepalming_tone1": "1f926-1f3fb",
    "person_lifting_weights_tone5": "1f3cb-1f3ff",
    "palms_up_together_tone3": "1f932-1f3fd",
    "man_red_haired_tone4": "1f468-1f3fe-1f9b0",
    "woman_artist_tone4": "1f469-1f3fe-1f3a8",
    "merman_tone4": "1f9dc-1f3fe-2642",
    "man_rowing_boat_tone4": "1f6a3-1f3fe-2642",
    "woman_tipping_hand_tone3": "1f481-1f3fd-2640",
    "man_gesturing_ok_tone2": "1f646-1f3fc-2642",
    "person_playing_water_polo_tone4": "1f93d-1f3fe",
    "man_bowing": "1f647-2642",
    "man_fairy_tone5": "1f9da-1f3ff-2642",
    "person_playing_handball_tone1": "1f93e-1f3fb",
    "man_juggling_tone2": "1f939-1f3fc-2642",
    "woman_gesturing_no_tone4": "1f645-1f3fe-2640",
    "flag_ea": "1f1ea-1f1e6",
    "man_bowing_tone1": "1f647-1f3fb-2642",
    "woman_vampire_tone5": "1f9db-1f3ff-2640",
    "badminton": "1f3f8",
    "woman_swimming_tone3": "1f3ca-1f3fd-2640",
    "older_adult_tone2": "1f9d3-1f3fc",
    "woman_running_tone3": "1f3c3-1f3fd-2640",
    "man_guard_tone1": "1f482-1f3fb-2642",
    "hand_splayed_tone2": "1f590-1f3fc",
    "man_vampire_tone3": "1f9db-1f3fd-2642",
    "man_tipping_hand": "1f481-2642",
    "man_supervillain_tone1": "1f9b9-1f3fb-2642",
    "family_man_boy_boy": "1f468-1f466-1f466",
    "flag_gr": "1f1ec-1f1f7",
    "point_left_tone2": "1f448-1f3fc",
    "man_with_chinese_cap_tone5": "1f472-1f3ff",
    "woman_singer_tone2": "1f469-1f3fc-1f3a4",
    "person_bowing_tone4": "1f647-1f3fe",
    "man_in_lotus_position_tone2": "1f9d8-1f3fc-2642",
    "man_swimming_tone2": "1f3ca-1f3fc-2642",
    "woman_rowing_boat_tone4": "1f6a3-1f3fe-2640",
    "man_with_chinese_cap_tone3": "1f472-1f3fd",
    "woman_gesturing_no_tone3": "1f645-1f3fd-2640",
    "man_cartwheeling_tone1": "1f938-1f3fb-2642",
    "flag_hu": "1f1ed-1f1fa",
    "woman_playing_water_polo_tone2": "1f93d-1f3fc-2640",
    "blond-haired_woman_tone2": "1f471-1f3fc-2640",
    "woman_facepalming_tone3": "1f926-1f3fd-2640",
    "man_cook_tone1": "1f468-1f3fb-1f373",
    "person_in_lotus_position_tone4": "1f9d8-1f3fe",
    "tone4": "1f3fe",
    "man_red_haired_tone3": "1f468-1f3fd-1f9b0",
    "man_biking_tone5": "1f6b4-1f3ff-2642",
    "police_officer_tone2": "1f46e-1f3fc",
    "flag_to": "1f1f9-1f1f4",
    "man_pilot_tone4": "1f468-1f3fe-2708",
    "man_pilot_tone1": "1f468-1f3fb-2708",
    "woman_technologist_tone3": "1f469-1f3fd-1f4bb",
    "flag_ms": "1f1f2-1f1f8",
    "t_rex": "1f996",
    "woman_health_worker_tone5": "1f469-1f3ff-2695",
    "flag_jp": "1f1ef-1f1f5",
    "fairy_tone2": "1f9da-1f3fc",
    "woman_bowing_tone5": "1f647-1f3ff-2640",
    "older_man_tone3": "1f474-1f3fd",
    "bath_tone1": "1f6c0-1f3fb",
    "mouse_three_button": "1f5b1",
    "woman_playing_handball": "1f93e-2640",
    "open_hands_tone3": "1f450-1f3fd",
    "raised_hand_tone3": "270b-1f3fd",
    "nail_care_tone5": "1f485-1f3ff",
    "flag_kg": "1f1f0-1f1ec",
    "raised_hand_tone5": "270b-1f3ff",
    "ok_hand_tone3": "1f44c-1f3fd",
    "older_man_tone2": "1f474-1f3fc",
    "white_sun_cloud": "1f325",
    "man_superhero_tone4": "1f9b8-1f3fe-2642",
    "person_climbing_tone3": "1f9d7-1f3fd",
    "woman_frowning_tone1": "1f64d-1f3fb-2640",
    "family_mmgg": "1f468-1f468-1f467-1f467",
    "flag_ck": "1f1e8-1f1f0",
    "flag_az": "1f1e6-1f1ff",
    "flag_kn": "1f1f0-1f1f3",
    "flag_za": "1f1ff-1f1e6",
    "man_farmer_tone4": "1f468-1f3fe-1f33e",
    "man_judge_tone1": "1f468-1f3fb-2696",
    "flag_tz": "1f1f9-1f1ff",
    "man_getting_haircut_tone2": "1f487-1f3fc-2642",
    "flag_ba": "1f1e7-1f1e6",
    "blond-haired_woman_tone5": "1f471-1f3ff-2640",
    "flag_at": "1f1e6-1f1f9",
    "woman_cook_tone2": "1f469-1f3fc-1f373",
    "flag_mo": "1f1f2-1f1f4",
    "person_shrugging_tone2": "1f937-1f3fc",
    "digit_five": "0035",
    "man_golfing_tone1": "1f3cc-1f3fb-2642",
    "man_teacher": "1f468-1f3eb",
    "man_curly_haired": "1f468-1f9b1",
    "woman_curly_haired_tone3": "1f469-1f3fd-1f9b1",
    "flag_kw": "1f1f0-1f1fc",
    "man_swimming": "1f3ca-2642",
    "person_doing_cartwheel_tone2": "1f938-1f3fc",
    "person_in_bed_tone3": "1f6cc-1f3fd",
    "bride_with_veil_tone5": "1f470-1f3ff",
    "person_climbing_tone4": "1f9d7-1f3fe",
    "older_man_tone5": "1f474-1f3ff",
    "person_golfing_tone5": "1f3cc-1f3ff",
    "foot_tone2": "1f9b6-1f3fc",
    "woman_bowing": "1f647-2640",
    "rofl": "1f923",
    "regional_indicator_h": "1f1ed",
    "horse_racing_tone5": "1f3c7-1f3ff",
    "pregnant_woman_tone2": "1f930-1f3fc",
    "woman_tone2": "1f469-1f3fc",
    "man_fairy_tone1": "1f9da-1f3fb-2642",
    "man_construction_worker": "1f477-2642",
    "person_raising_hand_tone4": "1f64b-1f3fe",
    "flag_cv": "1f1e8-1f1fb",
    "thumbsup_tone2": "1f44d-1f3fc",
    "flag_cr": "1f1e8-1f1f7",
    "flag_ec": "1f1ea-1f1e8",
    "santa_tone5": "1f385-1f3ff",
    "flag_lv": "1f1f1-1f1fb",
    "woman_frowning_tone4": "1f64d-1f3fe-2640",
    "man_singer_tone2": "1f468-1f3fc-1f3a4",
    "dancer_tone2": "1f483-1f3fc",
    "family_woman_girl_girl": "1f469-1f467-1f467",
    "squeeze_bottle": "1f9f4",
    "man_technologist_tone4": "1f468-1f3fe-1f4bb",
    "man_bald_tone4": "1f468-1f3fe-1f9b2",
    "woman_mechanic_tone3": "1f469-1f3fd-1f527",
    "levitate_tone1": "1f574-1f3fb",
    "man_getting_face_massage_tone5": "1f486-1f3ff-2642",
    "man_raising_hand": "1f64b-2642",
    "merperson_tone5": "1f9dc-1f3ff",
    "woman_juggling_tone5": "1f939-1f3ff-2640",
    "flag_uy": "1f1fa-1f1fe",
    "detective_tone5": "1f575-1f3ff",
    "man_singer_tone4": "1f468-1f3fe-1f3a4",
    "man_supervillain": "1f9b9-2642",
    "woman_white_haired": "1f469-1f9b3",
    "supervillain_tone3": "1f9b9-1f3fd",
    "flag_sg": "1f1f8-1f1ec",
    "woman_bald_tone2": "1f469-1f3fc-1f9b2",
    "love_you_gesture_tone5": "1f91f-1f3ff",
    "nerd": "1f913",
    "person_pouting_tone4": "1f64e-1f3fe",
    "blond-haired_woman_tone4": "1f471-1f3fe-2640",
    "muscle_tone3": "1f4aa-1f3fd",
    "mermaid_tone3": "1f9dc-1f3fd-2640",
    "woman_superhero_tone4": "1f9b8-1f3fe-2640",
    "menorah": "1f54e",
    "woman_with_headscarf_tone1": "1f9d5-1f3fb",
    "woman_guard_tone1": "1f482-1f3fb-2640",
    "man_mechanic_tone3": "1f468-1f3fd-1f527",
    "woman_juggling_tone1": "1f939-1f3fb-2640",
    "woman_lifting_weights_tone3": "1f3cb-1f3fd-2640",
    "flag_pk": "1f1f5-1f1f0",
    "flag_al": "1f1e6-1f1f1",
    "raised_back_of_hand_tone3": "1f91a-1f3fd",
    "love_you_gesture": "1f91f",
    "flag_tm": "1f1f9-1f1f2",
    "woman_surfing_tone2": "1f3c4-1f3fc-2640",
    "woman_in_steamy_room_tone5": "1f9d6-1f3ff-2640",
    "santa_tone4": "1f385-1f3fe",
    "flag_no": "1f1f3-1f1f4",
    "family_mmg": "1f468-1f468-1f467",
    "woman_mechanic_tone5": "1f469-1f3ff-1f527",
    "woman_singer_tone5": "1f469-1f3ff-1f3a4",
    "woman_in_lotus_position_tone3": "1f9d8-1f3fd-2640",
    "flag_sh": "1f1f8-1f1ed",
    "man_tipping_hand_tone5": "1f481-1f3ff-2642",
    "flag_nz": "1f1f3-1f1ff",
    "adult_tone3": "1f9d1-1f3fd",
    "dagger": "1f5e1",
    "flag_ar": "1f1e6-1f1f7",
    "man_detective_tone3": "1f575-1f3fd-2642",
    "man_facepalming_tone2": "1f926-1f3fc-2642",
    "person_running_tone1": "1f3c3-1f3fb",
    "milk": "1f95b",
    "woman_bouncing_ball_tone3": "26f9-1f3fd-2640",
    "man_rowing_boat_tone5": "1f6a3-1f3ff-2642",
    "woman_supervillain_tone5": "1f9b9-1f3ff-2640",
    "man_teacher_tone3": "1f468-1f3fd-1f3eb",
    "person_surfing_tone1": "1f3c4-1f3fb",
    "woman_biking_tone4": "1f6b4-1f3fe-2640",
    "woman_shrugging_tone5": "1f937-1f3ff-2640",
    "adult_tone1": "1f9d1-1f3fb",
    "older_adult_tone3": "1f9d3-1f3fd",
    "clap_tone1": "1f44f-1f3fb",
    "man_tone4": "1f468-1f3fe",
    "point_right_tone3": "1f449-1f3fd",
    "woman_cartwheeling": "1f938-2640",
    "person_mountain_biking_tone3": "1f6b5-1f3fd",
    "man_elf_tone2": "1f9dd-1f3fc-2642",
    "ok_hand_tone1": "1f44c-1f3fb",
    "woman_guard_tone5": "1f482-1f3ff-2640",
    "person_playing_water_polo": "1f93d",
    "leg_tone1": "1f9b5-1f3fb",
    "woman_pouting_tone5": "1f64e-1f3ff-2640",
    "person_lifting_weights_tone3": "1f3cb-1f3fd",
    "person_juggling_tone2": "1f939-1f3fc",
    "man_juggling": "1f939-2642",
    "flag_gi": "1f1ec-1f1ee",
    "man_elf_tone1": "1f9dd-1f3fb-2642",
    "flag_ai": "1f1e6-1f1ee",
    "man_astronaut": "1f468-1f680",
    "point_up_2_tone3": "1f446-1f3fd",
    "flag_hm": "1f1ed-1f1f2",
    "person_wearing_turban_tone1": "1f473-1f3fb",
    "face_with_symbols_over_mouth": "1f92c",
    "woman_mechanic": "1f469-1f527",
    "woman_swimming_tone2": "1f3ca-1f3fc-2640",
    "palms_up_together_tone4": "1f932-1f3fe",
    "flag_fk": "1f1eb-1f1f0",
    "dancer_tone5": "1f483-1f3ff",
    "call_me_tone4": "1f919-1f3fe",
    "woman_walking_tone4": "1f6b6-1f3fe-2640",
    "flag_nc": "1f1f3-1f1e8",
    "person_doing_cartwheel_tone1": "1f938-1f3fb",
    "breast_feeding_tone4": "1f931-1f3fe",
    "woman_climbing_tone2": "1f9d7-1f3fc-2640",
    "man_singer_tone1": "1f468-1f3fb-1f3a4",
    "man_judge_tone3": "1f468-1f3fd-2696",
    "man_vampire": "1f9db-2642",
    "man_firefighter": "1f468-1f692",
    "man_playing_water_polo_tone2": "1f93d-1f3fc-2642",
    "woman_firefighter_tone1": "1f469-1f3fb-1f692",
    "atom": "269b",
    "vampire_tone3": "1f9db-1f3fd",
    "man_vampire_tone1": "1f9db-1f3fb-2642",
    "person_in_steamy_room_tone1": "1f9d6-1f3fb",
    "leg_tone2": "1f9b5-1f3fc",
    "flag_bh": "1f1e7-1f1ed",
    "man_white_haired_tone3": "1f468-1f3fd-1f9b3",
    "person_playing_handball_tone2": "1f93e-1f3fc",
    "man_running_tone3": "1f3c3-1f3fd-2642",
    "blond-haired_man": "1f471-2642",
    "pen_ballpoint": "1f58a",
    "pregnant_woman_tone3": "1f930-1f3fd",
    "woman_frowning_tone3": "1f64d-1f3fd-2640",
    "nail_care_tone3": "1f485-1f3fd",
    "flag_td": "1f1f9-1f1e9",
    "person_juggling_tone3": "1f939-1f3fd",
    "person_pouting_tone5": "1f64e-1f3ff",
    "flag_mu": "1f1f2-1f1fa",
    "flag_yt": "1f1fe-1f1f9",
    "flag_is": "1f1ee-1f1f8",
    "woman_teacher_tone1": "1f469-1f3fb-1f3eb",
    "woman_bald_tone1": "1f469-1f3fb-1f9b2",
    "man_vampire_tone2": "1f9db-1f3fc-2642",
    "metal_tone3": "1f918-1f3fd",
    "fairy_tone3": "1f9da-1f3fd",
    "man_in_steamy_room_tone1": "1f9d6-1f3fb-2642",
    "woman_police_officer_tone1": "1f46e-1f3fb-2640",
    "man_mountain_biking_tone4": "1f6b5-1f3fe-2642",
    "flag_tk": "1f1f9-1f1f0",
    "v_tone2": "270c-1f3fc",
    "person_facepalming_tone2": "1f926-1f3fc",
    "nose_tone4": "1f443-1f3fe",
    "man_mage_tone4": "1f9d9-1f3fe-2642",
    "woman_cartwheeling_tone1": "1f938-1f3fb-2640",
    "person_facepalming_tone3": "1f926-1f3fd",
    "flag_so": "1f1f8-1f1f4",
    "flag_dj": "1f1e9-1f1ef",
    "person_rowing_boat": "1f6a3",
    "man_firefighter_tone2": "1f468-1f3fc-1f692",
    "snowboarder_tone2": "1f3c2-1f3fc",
    "man_health_worker_tone1": "1f468-1f3fb-2695",
    "flag_gu": "1f1ec-1f1fa",
    "house_abandoned": "1f3da",
    "man_gesturing_no_tone3": "1f645-1f3fd-2642",
    "man_curly_haired_tone4": "1f468-1f3fe-1f9b1",
    "third_place": "1f949",
    "people_with_bunny_ears_partying": "1f46f",
    "man_pouting_tone5": "1f64e-1f3ff-2642",
    "man_detective_tone5": "1f575-1f3ff-2642",
    "man_swimming_tone1": "1f3ca-1f3fb-2642",
    "point_left_tone4": "1f448-1f3fe",
    "woman_fairy_tone1": "1f9da-1f3fb-2640",
    "horse_racing_tone3": "1f3c7-1f3fd",
    "man_gesturing_ok_tone4": "1f646-1f3fe-2642",
    "writing_hand_tone1": "270d-1f3fb",
    "man_red_haired": "1f468-1f9b0",
    "man_rowing_boat": "1f6a3-2642",
    "woman_climbing_tone3": "1f9d7-1f3fd-2640",
    "man_mage": "1f9d9-2642",
    "flag_je": "1f1ef-1f1ea",
    "woman_astronaut_tone3": "1f469-1f3fd-1f680",
    "raised_hands_tone5": "1f64c-1f3ff",
    "family_woman_girl": "1f469-1f467",
    "person_gesturing_ok_tone2": "1f646-1f3fc",
    "man_surfing_tone5": "1f3c4-1f3ff-2642",
    "point_down_tone1": "1f447-1f3fb",
    "point_down_tone5": "1f447-1f3ff",
    "man_frowning_tone4": "1f64d-1f3fe-2642",
    "man_curly_haired_tone5": "1f468-1f3ff-1f9b1",
    "detective_tone4": "1f575-1f3fe",
    "man_gesturing_ok_tone1": "1f646-1f3fb-2642",
    "bellhop": "1f6ce",
    "woman_playing_handball_tone3": "1f93e-1f3fd-2640",
    "man_guard": "1f482-2642",
    "merperson_tone4": "1f9dc-1f3fe",
    "person_gesturing_ok_tone1": "1f646-1f3fb",
    "person_pouting_tone1": "1f64e-1f3fb",
    "woman_detective": "1f575-2640",
    "woman_shrugging_tone4": "1f937-1f3fe-2640",
    "man_mechanic_tone1": "1f468-1f3fb-1f527",
    "person_lifting_weights_tone2": "1f3cb-1f3fc",
    "man_bowing_tone5": "1f647-1f3ff-2642",
    "person_biking": "1f6b4",
    "woman_supervillain_tone4": "1f9b9-1f3fe-2640",
    "thumbsup_tone1": "1f44d-1f3fb",
    "thumbsup_tone5": "1f44d-1f3ff",
    "salad": "1f957",
    "girl_tone2": "1f467-1f3fc",
    "child_tone5": "1f9d2-1f3ff",
    "man_rowing_boat_tone1": "1f6a3-1f3fb-2642",
    "older_man_tone4": "1f474-1f3fe",
    "man_white_haired_tone1": "1f468-1f3fb-1f9b3",
    "pray_tone4": "1f64f-1f3fe",
    "man_technologist_tone5": "1f468-1f3ff-1f4bb",
    "woman_swimming": "1f3ca-2640",
    "point_right_tone2": "1f449-1f3fc",
    "person_mountain_biking_tone2": "1f6b5-1f3fc",
    "person_golfing_tone2": "1f3cc-1f3fc",
    "palms_up_together_tone2": "1f932-1f3fc",
    "woman_police_officer_tone4": "1f46e-1f3fe-2640",
    "woman_bouncing_ball_tone5": "26f9-1f3ff-2640",
    "man_vampire_tone5": "1f9db-1f3ff-2642",
    "guard_tone2": "1f482-1f3fc",
    "woman_police_officer": "1f46e-2640",
    "woman_office_worker_tone2": "1f469-1f3fc-1f4bc",
    "person_playing_handball_tone4": "1f93e-1f3fe",
    "man_farmer_tone3": "1f468-1f3fd-1f33e",
    "thermometer_face": "1f912",
    "woman_health_worker_tone1": "1f469-1f3fb-2695",
    "regional_indicator_b": "1f1e7",
    "person_climbing_tone1": "1f9d7-1f3fb",
    "ok_hand_tone2": "1f44c-1f3fc",
    "leg_tone3": "1f9b5-1f3fd",
    "woman_swimming_tone1": "1f3ca-1f3fb-2640",
    "woman_office_worker": "1f469-1f4bc",
    "man_firefighter_tone1": "1f468-1f3fb-1f692",
    "man_cartwheeling_tone2": "1f938-1f3fc-2642",
    "paperclips": "1f587",
    "woman_bald_tone3": "1f469-1f3fd-1f9b2",
    "crayon": "1f58d",
    "person_raising_hand_tone1": "1f64b-1f3fb",
    "man_genie": "1f9de-2642",
    "man_walking_tone5": "1f6b6-1f3ff-2642",
    "unicorn": "1f984",
    "man_lifting_weights_tone3": "1f3cb-1f3fd-2642",
    "blond_haired_person_tone2": "1f471-1f3fc",
    "man_guard_tone2": "1f482-1f3fc-2642",
    "woman_police_officer_tone5": "1f46e-1f3ff-2640",
    "man_pouting_tone4": "1f64e-1f3fe-2642",
    "man_gesturing_ok_tone5": "1f646-1f3ff-2642",
    "flag_au": "1f1e6-1f1fa",
    "merperson_tone2": "1f9dc-1f3fc",
    "point_down_tone2": "1f447-1f3fc",
    "person_bowing": "1f647",
    "man_supervillain_tone2": "1f9b9-1f3fc-2642",
    "woman_bowing_tone3": "1f647-1f3fd-2640",
    "man_gesturing_no_tone1": "1f645-1f3fb-2642",
    "man_raising_hand_tone3": "1f64b-1f3fd-2642",
    "woman_golfing_tone5": "1f3cc-1f3ff-2640",
    "frowning2": "2639",
    "person_biking_tone3": "1f6b4-1f3fd",
    "person_walking_tone4": "1f6b6-1f3fe",
    "flag_vi": "1f1fb-1f1ee",
    "cowboy": "1f920",
    "woman_scientist": "1f469-1f52c",
    "fist_tone5": "270a-1f3ff",
    "woman_swimming_tone5": "1f3ca-1f3ff-2640",
    "flag_ht": "1f1ed-1f1f9",
    "pray_tone1": "1f64f-1f3fb",
    "woman_artist_tone5": "1f469-1f3ff-1f3a8",
    "princess_tone2": "1f478-1f3fc",
    "woman_artist": "1f469-1f3a8",
    "woman_artist_tone3": "1f469-1f3fd-1f3a8",
    "woman_guard_tone4": "1f482-1f3fe-2640",
    "woman_walking_tone2": "1f6b6-1f3fc-2640",
    "flag_et": "1f1ea-1f1f9",
    "flag_tw": "1f1f9-1f1fc",
    "boy_tone5": "1f466-1f3ff",
    "woman_pilot_tone4": "1f469-1f3fe-2708",
    "woman_rowing_boat_tone5": "1f6a3-1f3ff-2640",
    "woman_curly_haired_tone2": "1f469-1f3fc-1f9b1",
    "point_up_2_tone2": "1f446-1f3fc",
    "flag_tj": "1f1f9-1f1ef",
    "person_getting_haircut_tone2": "1f487-1f3fc",
    "flag_io": "1f1ee-1f1f4",
    "man_golfing_tone2": "1f3cc-1f3fc-2642",
    "blue_circle": "1f535",
    "regional_indicator_s": "1f1f8",
    "man_mechanic_tone2": "1f468-1f3fc-1f527",
    "thumbsdown_tone5": "1f44e-1f3ff",
    "flag_mn": "1f1f2-1f1f3",
    "flag_ma": "1f1f2-1f1e6",
    "thumbsup_tone3": "1f44d-1f3fd",
    "speaking_head": "1f5e3",
    "flag_do": "1f1e9-1f1f4",
    "woman_pouting": "1f64e-2640",
    "flag_white": "1f3f3",
    "man_farmer": "1f468-1f33e",
    "clock": "1f570",
    "woman_surfing_tone4": "1f3c4-1f3fe-2640",
    "hand_splayed_tone5": "1f590-1f3ff",
    "woman_factory_worker_tone4": "1f469-1f3fe-1f3ed",
    "man_walking_tone1": "1f6b6-1f3fb-2642",
    "flag_gm": "1f1ec-1f1f2",
    "woman_mage_tone4": "1f9d9-1f3fe-2640",
    "blond_haired_person_tone5": "1f471-1f3ff",
    "men_with_bunny_ears_partying": "1f46f-2642",
    "woman_vampire": "1f9db-2640",
    "flag_ir": "1f1ee-1f1f7",
    "woman_scientist_tone3": "1f469-1f3fd-1f52c",
    "man_bowing_tone2": "1f647-1f3fc-2642",
    "prince_tone1": "1f934-1f3fb",
    "man_getting_haircut": "1f487-2642",
    "nose_tone5": "1f443-1f3ff",
    "fairy_tone4": "1f9da-1f3fe",
    "woman_golfing": "1f3cc-2640",
    "record_button": "23fa",
    "person_in_steamy_room_tone3": "1f9d6-1f3fd",
    "thumbsdown_tone2": "1f44e-1f3fc",
    "woman_guard_tone3": "1f482-1f3fd-2640",
    "fleur-de-lis": "269c",
    "woman_construction_worker_tone1": "1f477-1f3fb-2640",
    "woman_fairy_tone4": "1f9da-1f3fe-2640",
    "raised_hands_tone4": "1f64c-1f3fe",
    "man_getting_face_massage": "1f486-2642",
    "cross": "271d",
    "white_sun_small_cloud": "1f324",
    "man_frowning_tone5": "1f64d-1f3ff-2642",
    "person_gesturing_ok_tone4": "1f646-1f3fe",
    "flag_tv": "1f1f9-1f1fb",
    "bath_tone2": "1f6c0-1f3fc",
    "umbrella2": "2602",
    "woman_technologist_tone5": "1f469-1f3ff-1f4bb",
    "couple_with_heart_woman_man": "1f469-2764-1f468",
    "frame_photo": "1f5bc",
    "projector": "1f4fd",
    "woman_shrugging_tone3": "1f937-1f3fd-2640",
    "woman_astronaut_tone4": "1f469-1f3fe-1f680",
    "flag_ua": "1f1fa-1f1e6",
    "person_swimming_tone5": "1f3ca-1f3ff",
    "person_running_tone3": "1f3c3-1f3fd",
    "hand_splayed_tone3": "1f590-1f3fd",
    "woman_tipping_hand_tone1": "1f481-1f3fb-2640",
    "nail_care_tone1": "1f485-1f3fb",
    "first_place": "1f947",
    "flag_ng": "1f1f3-1f1ec",
    "woman_gesturing_ok_tone4": "1f646-1f3fe-2640",
    "woman_elf_tone3": "1f9dd-1f3fd-2640",
    "flag_tt": "1f1f9-1f1f9",
    "raised_back_of_hand_tone5": "1f91a-1f3ff",
    "woman_factory_worker_tone2": "1f469-1f3fc-1f3ed",
    "man_factory_worker_tone1": "1f468-1f3fb-1f3ed",
    "girl_tone4": "1f467-1f3fe",
    "flag_as": "1f1e6-1f1f8",
    "flag_mm": "1f1f2-1f1f2",
    "couch": "1f6cb",
    "digit_two": "0032",
    "person_climbing_tone2": "1f9d7-1f3fc",
    "woman_student_tone5": "1f469-1f3ff-1f393",
    "man_cartwheeling": "1f938-2642",
    "supervillain_tone5": "1f9b9-1f3ff",
    "shopping_cart": "1f6d2",
    "woman_pilot_tone1": "1f469-1f3fb-2708",
    "person_bowing_tone2": "1f647-1f3fc",
    "flag_cg": "1f1e8-1f1ec",
    "writing_hand_tone2": "270d-1f3fc",
    "bath_tone3": "1f6c0-1f3fd",
    "united_nations": "1f1fa-1f1f3",
    "flag_ta": "1f1f9-1f1e6",
    "nose_tone1": "1f443-1f3fb",
    "woman_gesturing_no_tone2": "1f645-1f3fc-2640",
    "person_in_lotus_position_tone1": "1f9d8-1f3fb",
    "flag_ic": "1f1ee-1f1e8",
    "mermaid_tone1": "1f9dc-1f3fb-2640",
    "hand_splayed_tone4": "1f590-1f3fe",
    "woman_guard_tone2": "1f482-1f3fc-2640",
    "regional_indicator_k": "1f1f0",
    "flag_um": "1f1fa-1f1f2",
    "flag_il": "1f1ee-1f1f1",
    "woman_golfing_tone4": "1f3cc-1f3fe-2640",
    "woman_biking": "1f6b4-2640",
    "woman_technologist_tone4": "1f469-1f3fe-1f4bb",
    "breast_feeding": "1f931",
    "man_surfing_tone2": "1f3c4-1f3fc-2642",
    "nose_tone3": "1f443-1f3fd",
    "flag_cz": "1f1e8-1f1ff",
    "man_red_haired_tone2": "1f468-1f3fc-1f9b0",
    "man_office_worker_tone1": "1f468-1f3fb-1f4bc",
    "person_gesturing_ok": "1f646",
    "levitate": "1f574",
    "superhero_tone5": "1f9b8-1f3ff",
    "man_factory_worker_tone5": "1f468-1f3ff-1f3ed",
    "flag_sl": "1f1f8-1f1f1",
    "selfie_tone4": "1f933-1f3fe",
    "man_mountain_biking": "1f6b5-2642",
    "woman_mage_tone3": "1f9d9-1f3fd-2640",
    "woman_firefighter": "1f469-1f692",
    "flag_mh": "1f1f2-1f1ed",
    "person_gesturing_no_tone5": "1f645-1f3ff",
    "park": "1f3de",
    "right_facing_fist_tone5": "1f91c-1f3ff",
    "man_gesturing_no": "1f645-2642",
    "man_scientist_tone2": "1f468-1f3fc-1f52c",
    "woman_with_headscarf_tone3": "1f9d5-1f3fd",
    "woman_astronaut": "1f469-1f680",
    "woman_tipping_hand": "1f481-2640",
    "elf_tone5": "1f9dd-1f3ff",
    "woman_farmer_tone1": "1f469-1f3fb-1f33e",
    "woman_teacher": "1f469-1f3eb",
    "man_technologist": "1f468-1f4bb",
    "punch_tone2": "1f44a-1f3fc",
    "women_with_bunny_ears_partying": "1f46f-2640",
    "flag_jm": "1f1ef-1f1f2",
    "flag_ss": "1f1f8-1f1f8",
    "person_gesturing_ok_tone3": "1f646-1f3fd",
    "woman_playing_handball_tone5": "1f93e-1f3ff-2640",
    "man_factory_worker": "1f468-1f3ed",
    "woman_climbing_tone4": "1f9d7-1f3fe-2640",
    "man_biking_tone3": "1f6b4-1f3fd-2642",
    "flag_cf": "1f1e8-1f1eb",
    "flag_sz": "1f1f8-1f1ff",
    "man_in_tuxedo_tone2": "1f935-1f3fc",
    "flag_sj": "1f1f8-1f1ef",
    "woman_wearing_turban_tone4": "1f473-1f3fe-2640",
    "woman_artist_tone1": "1f469-1f3fb-1f3a8",
    "man_rowing_boat_tone2": "1f6a3-1f3fc-2642",
    "man_student_tone2": "1f468-1f3fc-1f393",
    "punch_tone3": "1f44a-1f3fd",
    "metal_tone4": "1f918-1f3fe",
    "woman_playing_water_polo_tone3": "1f93d-1f3fd-2640",
    "regional_indicator_d": "1f1e9",
    "pen_fountain": "1f58b",
    "flag_rw": "1f1f7-1f1fc",
    "woman_supervillain_tone3": "1f9b9-1f3fd-2640",
    "regional_indicator_o": "1f1f4",
    "flag_gb": "1f1ec-1f1e7",
    "middle_finger_tone3": "1f595-1f3fd",
    "man_shrugging_tone1": "1f937-1f3fb-2642",
    "flag_aw": "1f1e6-1f1fc",
    "flag_re": "1f1f7-1f1ea",
    "flag_mq": "1f1f2-1f1f6",
    "man_pilot_tone5": "1f468-1f3ff-2708",
    "man_cook_tone2": "1f468-1f3fc-1f373",
    "flag_hn": "1f1ed-1f1f3",
    "man_police_officer": "1f46e-2642",
    "blond_haired_person_tone1": "1f471-1f3fb",
    "woman_judge_tone4": "1f469-1f3fe-2696",
    "flag_vn": "1f1fb-1f1f3",
    "fist_tone1": "270a-1f3fb",
    "flag_bm": "1f1e7-1f1f2",
    "woman_shrugging": "1f937-2640",
    "woman_getting_haircut_tone4": "1f487-1f3fe-2640",
    "person_shrugging_tone1": "1f937-1f3fb",
    "flag_sx": "1f1f8-1f1fd",
    "man_facepalming_tone4": "1f926-1f3fe-2642",
    "older_adult_tone1": "1f9d3-1f3fb",
    "man_teacher_tone5": "1f468-1f3ff-1f3eb",
    "man_technologist_tone3": "1f468-1f3fd-1f4bb",
    "woman_singer_tone1": "1f469-1f3fb-1f3a4",
    "dancer_tone1": "1f483-1f3fb",
    "flag_ru": "1f1f7-1f1fa",
    "right_facing_fist_tone1": "1f91c-1f3fb",
    "family_mmbb": "1f468-1f468-1f466-1f466",
    "woman_bowing_tone4": "1f647-1f3fe-2640",
    "hammer_pick": "2692",
    "woman_climbing_tone5": "1f9d7-1f3ff-2640",
    "man_frowning_tone3": "1f64d-1f3fd-2642",
    "flag_ke": "1f1f0-1f1ea",
    "flag_cu": "1f1e8-1f1fa",
    "person_walking_tone5": "1f6b6-1f3ff",
    "horse_racing_tone1": "1f3c7-1f3fb",
    "person_wearing_turban_tone3": "1f473-1f3fd",
    "woman_frowning_tone2": "1f64d-1f3fc-2640",
    "man_student_tone4": "1f468-1f3fe-1f393",
    "man_facepalming_tone5": "1f926-1f3ff-2642",
    "satellite_orbital": "1f6f0",
    "woman_red_haired_tone2": "1f469-1f3fc-1f9b0",
    "flag_vg": "1f1fb-1f1ec",
    "man_mage_tone1": "1f9d9-1f3fb-2642",
    "woman_elf_tone2": "1f9dd-1f3fc-2640",
    "merperson_tone1": "1f9dc-1f3fb",
    "thunder_cloud_rain": "26c8",
    "regional_indicator_r": "1f1f7",
    "woman_shrugging_tone2": "1f937-1f3fc-2640",
    "man_gesturing_ok_tone3": "1f646-1f3fd-2642",
    "breast_feeding_tone2": "1f931-1f3fc",
    "man_cartwheeling_tone3": "1f938-1f3fd-2642",
    "breast_feeding_tone3": "1f931-1f3fd",
    "man_astronaut_tone4": "1f468-1f3fe-1f680",
    "person_golfing_tone4": "1f3cc-1f3fe",
    "woman_swimming_tone4": "1f3ca-1f3fe-2640",
    "man_firefighter_tone4": "1f468-1f3fe-1f692",
    "woman_tone3": "1f469-1f3fd",
    "regional_indicator_u": "1f1fa",
    "head_bandage": "1f915",
    "woman_cook_tone4": "1f469-1f3fe-1f373",
    "man_playing_water_polo_tone1": "1f93d-1f3fb-2642",
    "flag_ne": "1f1f3-1f1ea",
    "flag_zw": "1f1ff-1f1fc",
    "angel_tone4": "1f47c-1f3fe",
    "flag_sa": "1f1f8-1f1e6",
    "man_in_lotus_position_tone4": "1f9d8-1f3fe-2642",
    "construction_worker_tone4": "1f477-1f3fe",
    "tickets": "1f39f",
    "flag_cl": "1f1e8-1f1f1",
    "flag_ca": "1f1e8-1f1e6",
    "flag_pr": "1f1f5-1f1f7",
    "fairy_tone5": "1f9da-1f3ff",
    "man_curly_haired_tone2": "1f468-1f3fc-1f9b1",
    "drum": "1f941",
    "vulcan_tone5": "1f596-1f3ff",
    "woman_office_worker_tone5": "1f469-1f3ff-1f4bc",
    "woman_vampire_tone3": "1f9db-1f3fd-2640",
    "angel_tone2": "1f47c-1f3fc",
    "tone3": "1f3fd",
    "person_raising_hand": "1f64b",
    "hand_splayed": "1f590",
    "flag_mc": "1f1f2-1f1e8",
    "white_sun_rain_cloud": "1f326",
    "wave_tone5": "1f44b-1f3ff",
    "point_left_tone5": "1f448-1f3ff",
    "man_teacher_tone1": "1f468-1f3fb-1f3eb",
    "speech_left": "1f5e8",
    "woman_red_haired_tone4": "1f469-1f3fe-1f9b0",
    "man_scientist_tone4": "1f468-1f3fe-1f52c",
    "woman_farmer_tone3": "1f469-1f3fd-1f33e",
    "woman_surfing_tone3": "1f3c4-1f3fd-2640",
    "man_police_officer_tone1": "1f46e-1f3fb-2642",
    "digit_three": "0033",
    "woman_supervillain": "1f9b9-2640",
    "woman_health_worker": "1f469-2695",
    "man_facepalming": "1f926-2642",
    "woman_cartwheeling_tone5": "1f938-1f3ff-2640",
    "woman_elf": "1f9dd-2640",
    "woman_cook_tone3": "1f469-1f3fd-1f373",
    "couple_mm": "1f468-2764-1f468",
    "woman_fairy_tone3": "1f9da-1f3fd-2640",
    "digit_eight": "0038",
    "flag_cd": "1f1e8-1f1e9",
    "family_woman_girl_boy": "1f469-1f467-1f466",
    "flag_nr": "1f1f3-1f1f7",
    "flag_lk": "1f1f1-1f1f0",
    "person_facepalming_tone5": "1f926-1f3ff",
    "man_construction_worker_tone1": "1f477-1f3fb-2642",
    "person_wearing_turban_tone4": "1f473-1f3fe",
    "man_tone5": "1f468-1f3ff",
    "woman_pouting_tone1": "1f64e-1f3fb-2640",
    "flag_vc": "1f1fb-1f1e8",
    "woman_in_lotus_position_tone5": "1f9d8-1f3ff-2640",
    "digit_one": "0031",
    "cloud_tornado": "1f32a",
    "flag_bj": "1f1e7-1f1ef",
    "mermaid_tone2": "1f9dc-1f3fc-2640",
    "woman_cook": "1f469-1f373",
    "flag_pl": "1f1f5-1f1f1",
    "thumbsdown_tone1": "1f44e-1f3fb",
    "man_shrugging_tone2": "1f937-1f3fc-2642",
    "woman_rowing_boat": "1f6a3-2640",
    "woman_teacher_tone4": "1f469-1f3fe-1f3eb",
    "ear_tone5": "1f442-1f3ff",
    "woman_supervillain_tone1": "1f9b9-1f3fb-2640",
    "man_farmer_tone2": "1f468-1f3fc-1f33e",
    "woman_farmer_tone5": "1f469-1f3ff-1f33e",
    "man_elf": "1f9dd-2642",
    "man_playing_water_polo_tone4": "1f93d-1f3fe-2642",
    "thumbsup_tone4": "1f44d-1f3fe",
    "person_shrugging_tone4": "1f937-1f3fe",
    "flag_dm": "1f1e9-1f1f2",
    "detective_tone3": "1f575-1f3fd",
    "man_artist_tone5": "1f468-1f3ff-1f3a8",
    "flag_sy": "1f1f8-1f1fe",
    "man_cook_tone5": "1f468-1f3ff-1f373",
    "rhino": "1f98f",
    "prince_tone3": "1f934-1f3fd",
    "woman_with_headscarf_tone2": "1f9d5-1f3fc",
    "bearded_person_tone2": "1f9d4-1f3fc",
    "flag_np": "1f1f3-1f1f5",
    "raised_hand_tone2": "270b-1f3fc",
    "woman_factory_worker_tone1": "1f469-1f3fb-1f3ed",
    "woman_gesturing_no": "1f645-2640",
    "man_surfing_tone1": "1f3c4-1f3fb-2642",
    "baby_tone3": "1f476-1f3fd",
    "police_officer_tone5": "1f46e-1f3ff",
    "woman_mountain_biking_tone1": "1f6b5-1f3fb-2640",
    "blond-haired_woman_tone3": "1f471-1f3fd-2640",
    "snowman2": "2603",
    "person_pouting_tone2": "1f64e-1f3fc",
    "person_biking_tone4": "1f6b4-1f3fe",
    "man_singer_tone3": "1f468-1f3fd-1f3a4",
    "man_playing_handball_tone1": "1f93e-1f3fb-2642",
    "vampire_tone5": "1f9db-1f3ff",
    "boy_tone3": "1f466-1f3fd",
    "man_pilot": "1f468-2708",
    "person_tipping_hand_tone3": "1f481-1f3fd",
    "dancer_tone3": "1f483-1f3fd",
    "levitate_tone2": "1f574-1f3fc",
    "family_man_woman_boy": "1f468-1f469-1f466",
    "flag_ls": "1f1f1-1f1f8",
    "man_construction_worker_tone3": "1f477-1f3fd-2642",
    "man_in_steamy_room_tone5": "1f9d6-1f3ff-2642",
    "merman_tone2": "1f9dc-1f3fc-2642",
    "prince_tone5": "1f934-1f3ff",
    "punch_tone1": "1f44a-1f3fb",
    "man_vampire_tone4": "1f9db-1f3fe-2642",
    "woman_tipping_hand_tone4": "1f481-1f3fe-2640",
    "woman_surfing": "1f3c4-2640",
    "person_getting_haircut_tone5": "1f487-1f3ff",
    "man_mage_tone2": "1f9d9-1f3fc-2642",
    "man_lifting_weights_tone4": "1f3cb-1f3fe-2642",
    "man_scientist_tone1": "1f468-1f3fb-1f52c",
    "flag_ml": "1f1f2-1f1f1",
    "left_facing_fist_tone2": "1f91b-1f3fc",
    "flag_sb": "1f1f8-1f1e7",
    "biohazard": "2623",
    "flag_fj": "1f1eb-1f1ef",
    "man_white_haired_tone4": "1f468-1f3fe-1f9b3",
    "woman_bald": "1f469-1f9b2",
    "person_biking_tone2": "1f6b4-1f3fc",
    "woman_gesturing_ok_tone1": "1f646-1f3fb-2640",
    "person_golfing": "1f3cc",
    "man_teacher_tone2": "1f468-1f3fc-1f3eb",
    "woman_police_officer_tone3": "1f46e-1f3fd-2640",
    "man_dancing_tone5": "1f57a-1f3ff",
    "person_running_tone5": "1f3c3-1f3ff",
    "left_facing_fist_tone4": "1f91b-1f3fe",
    "woman_white_haired_tone5": "1f469-1f3ff-1f9b3",
    "woman_in_steamy_room_tone1": "1f9d6-1f3fb-2640",
    "family_man_boy": "1f468-1f466",
    "regional_indicator_g": "1f1ec",
    "flag_kz": "1f1f0-1f1ff",
    "woman_running": "1f3c3-2640",
    "woman_cook_tone5": "1f469-1f3ff-1f373",
    "person_getting_massage_tone5": "1f486-1f3ff",
    "woman_biking_tone5": "1f6b4-1f3ff-2640",
    "man_biking": "1f6b4-2642",
    "supervillain_tone4": "1f9b9-1f3fe",
    "foot_tone1": "1f9b6-1f3fb",
    "man_raising_hand_tone1": "1f64b-1f3fb-2642",
    "flag_ug": "1f1fa-1f1ec",
    "blond-haired_man_tone2": "1f471-1f3fc-2642",
    "flag_be": "1f1e7-1f1ea",
    "man_dancing_tone3": "1f57a-1f3fd",
    "middle_finger_tone1": "1f595-1f3fb",
    "older_woman_tone4": "1f475-1f3fe",
    "man_bald_tone3": "1f468-1f3fd-1f9b2",
    "man_wearing_turban_tone1": "1f473-1f3fb-2642",
    "person_mountain_biking_tone4": "1f6b5-1f3fe",
    "point_up_tone1": "261d-1f3fb",
    "kiwi": "1f95d",
    "call_me_tone5": "1f919-1f3ff",
    "flag_ao": "1f1e6-1f1f4",
    "raised_back_of_hand_tone2": "1f91a-1f3fc",
    "flag_cw": "1f1e8-1f1fc",
    "right_facing_fist_tone3": "1f91c-1f3fd",
    "flag_pw": "1f1f5-1f1fc",
    "man_raising_hand_tone2": "1f64b-1f3fc-2642",
    "older_woman_tone3": "1f475-1f3fd",
    "man_playing_water_polo_tone5": "1f93d-1f3ff-2642",
    "woman_running_tone4": "1f3c3-1f3fe-2640",
    "digit_four": "0034",
    "flag_id": "1f1ee-1f1e9",
    "man_superhero_tone3": "1f9b8-1f3fd-2642",
    "man_gesturing_no_tone2": "1f645-1f3fc-2642",
    "woman_office_worker_tone3": "1f469-1f3fd-1f4bc",
    "family_mwgg": "1f468-1f469-1f467-1f467",
    "man_getting_haircut_tone3": "1f487-1f3fd-2642",
    "regional_indicator_q": "1f1f6",
    "baby_tone1": "1f476-1f3fb",
    "woman_pilot_tone3": "1f469-1f3fd-2708",
    "flag_iq": "1f1ee-1f1f6",
    "flag_mx": "1f1f2-1f1fd",
    "flag_pn": "1f1f5-1f1f3",
    "flag_py": "1f1f5-1f1fe",
    "man_surfing": "1f3c4-2642",
    "man_superhero_tone5": "1f9b8-1f3ff-2642",
    "man_fairy_tone4": "1f9da-1f3fe-2642",
    "middle_finger_tone4": "1f595-1f3fe",
    "person_frowning_tone3": "1f64d-1f3fd",
    "mrs_claus_tone4": "1f936-1f3fe",
    "digit_zero": "0030",
    "man_student": "1f468-1f393",
    "man_mountain_biking_tone3": "1f6b5-1f3fd-2642",
    "paintbrush": "1f58c",
    "ok_hand_tone4": "1f44c-1f3fe",
    "flag_gq": "1f1ec-1f1f6",
    "woman_playing_handball_tone4": "1f93e-1f3fe-2640",
    "mage_tone2": "1f9d9-1f3fc",
    "flag_mp": "1f1f2-1f1f5",
    "man_raising_hand_tone5": "1f64b-1f3ff-2642",
    "person_raising_hand_tone2": "1f64b-1f3fc",
    "man_curly_haired_tone3": "1f468-1f3fd-1f9b1",
    "scotland": "1f3f4-e0067-e0062-e0073-e0063-e0074-e007f",
    "blond_haired_person_tone3": "1f471-1f3fd",
    "flag_nf": "1f1f3-1f1eb",
    "raised_hands_tone1": "1f64c-1f3fb",
    "woman_curly_haired_tone5": "1f469-1f3ff-1f9b1",
    "cloud_lightning": "1f329",
    "men_wrestling": "1f93c-2642",
    "wave_tone3": "1f44b-1f3fd",
    "girl_tone3": "1f467-1f3fd",
    "man_gesturing_no_tone5": "1f645-1f3ff-2642",
    "person_getting_haircut_tone3": "1f487-1f3fd",
    "guard": "1f482",
    "snowboarder_tone5": "1f3c2-1f3ff",
    "bath_tone4": "1f6c0-1f3fe",
    "man_frowning_tone1": "1f64d-1f3fb-2642",
    "muscle_tone5": "1f4aa-1f3ff",
    "person_surfing_tone3": "1f3c4-1f3fd",
    "woman_mountain_biking_tone4": "1f6b5-1f3fe-2640",
    "man_cook_tone3": "1f468-1f3fd-1f373",
    "woman_facepalming_tone4": "1f926-1f3fe-2640",
    "man_rowing_boat_tone3": "1f6a3-1f3fd-2642",
    "woman_elf_tone5": "1f9dd-1f3ff-2640",
    "flag_pm": "1f1f5-1f1f2",
    "woman_raising_hand_tone2": "1f64b-1f3fc-2640",
    "calendar_spiral": "1f5d3",
    "man_pouting_tone3": "1f64e-1f3fd-2642",
    "flag_dz": "1f1e9-1f1ff",
    "man_shrugging": "1f937-2642",
    "man_astronaut_tone3": "1f468-1f3fd-1f680",
    "mage_tone3": "1f9d9-1f3fd",
    "person_frowning_tone5": "1f64d-1f3ff",
    "woman_mage": "1f9d9-2640",
    "woman_firefighter_tone5": "1f469-1f3ff-1f692",
    "elf_tone4": "1f9dd-1f3fe",
    "man_student_tone3": "1f468-1f3fd-1f393",
    "flag_sn": "1f1f8-1f1f3",
    "man_office_worker_tone5": "1f468-1f3ff-1f4bc",
    "man_artist_tone4": "1f468-1f3fe-1f3a8",
    "man_walking_tone2": "1f6b6-1f3fc-2642",
    "woman_with_headscarf": "1f9d5",
    "person_lifting_weights": "1f3cb",
    "fist_tone3": "270a-1f3fd",
    "bearded_person_tone5": "1f9d4-1f3ff",
    "woman_getting_haircut_tone5": "1f487-1f3ff-2640",
    "person_bouncing_ball": "26f9",
    "right_facing_fist_tone4": "1f91c-1f3fe",
    "left_facing_fist": "1f91b",
    "superhero_tone1": "1f9b8-1f3fb",
    "flag_bl": "1f1e7-1f1f1",
    "muscle_tone4": "1f4aa-1f3fe",
    "person_doing_cartwheel_tone5": "1f938-1f3ff",
    "woman_zombie": "1f9df-2640",
    "vulcan_tone3": "1f596-1f3fd",
    "man_running_tone4": "1f3c3-1f3fe-2642",
    "flag_bo": "1f1e7-1f1f4",
    "family_wwgg": "1f469-1f469-1f467-1f467",
    "family_mmb": "1f468-1f468-1f466",
    "champagne_glass": "1f942",
    "track_next": "23ed",
    "levitate_tone5": "1f574-1f3ff",
    "rainbow_flag": "1f3f3-1f308",
    "person_biking_tone1": "1f6b4-1f3fb",
    "person_lifting_weights_tone4": "1f3cb-1f3fe",
    "person_in_lotus_position_tone3": "1f9d8-1f3fd",
    "pray_tone2": "1f64f-1f3fc",
    "police_officer_tone3": "1f46e-1f3fd",
    "woman_pilot_tone2": "1f469-1f3fc-2708",
    "mage_tone1": "1f9d9-1f3fb",
    "man_scientist_tone3": "1f468-1f3fd-1f52c",
    "man_supervillain_tone5": "1f9b9-1f3ff-2642",
    "man_getting_face_massage_tone1": "1f486-1f3fb-2642",
    "woman_red_haired_tone5": "1f469-1f3ff-1f9b0",
    "woman_construction_worker": "1f477-2640",
    "man_playing_handball_tone4": "1f93e-1f3fe-2642",
    "man_playing_handball_tone3": "1f93e-1f3fd-2642",
    "person_rowing_boat_tone2": "1f6a3-1f3fc",
    "man_playing_handball": "1f93e-2642",
    "woman_bowing_tone2": "1f647-1f3fc-2640",
    "fingers_crossed_tone4": "1f91e-1f3fe",
    "hand_splayed_tone1": "1f590-1f3fb",
    "elf_tone2": "1f9dd-1f3fc",
    "person_playing_handball_tone3": "1f93e-1f3fd",
    "woman_white_haired_tone4": "1f469-1f3fe-1f9b3",
    "man_with_chinese_cap_tone2": "1f472-1f3fc",
    "construction_worker_tone2": "1f477-1f3fc",
    "man_running_tone1": "1f3c3-1f3fb-2642",
    "flag_sk": "1f1f8-1f1f0",
    "woman_walking_tone3": "1f6b6-1f3fd-2640",
    "woman_office_worker_tone1": "1f469-1f3fb-1f4bc",
    "upside_down": "1f643",
    "flag_lt": "1f1f1-1f1f9",
    "man_detective_tone1": "1f575-1f3fb-2642",
    "fist_tone4": "270a-1f3fe",
    "man_wearing_turban_tone5": "1f473-1f3ff-2642",
    "woman_health_worker_tone3": "1f469-1f3fd-2695",
    "person_getting_massage_tone2": "1f486-1f3fc",
    "man_police_officer_tone2": "1f46e-1f3fc-2642",
    "person_shrugging_tone5": "1f937-1f3ff",
    "woman_running_tone5": "1f3c3-1f3ff-2640",
    "flag_va": "1f1fb-1f1e6",
    "man_office_worker": "1f468-1f4bc",
    "man_running_tone2": "1f3c3-1f3fc-2642",
    "flag_rs": "1f1f7-1f1f8",
    "woman_getting_face_massage_tone5": "1f486-1f3ff-2640",
    "boy_tone2": "1f466-1f3fc",
    "man_in_tuxedo_tone5": "1f935-1f3ff",
    "man_playing_handball_tone2": "1f93e-1f3fc-2642",
    "woman_wearing_turban_tone2": "1f473-1f3fc-2640",
    "woman_bald_tone4": "1f469-1f3fe-1f9b2",
    "flag_bf": "1f1e7-1f1eb",
    "flag_ad": "1f1e6-1f1e9",
    "flag_eh": "1f1ea-1f1ed",
    "woman_scientist_tone4": "1f469-1f3fe-1f52c",
    "flag_im": "1f1ee-1f1f2",
    "woman_rowing_boat_tone2": "1f6a3-1f3fc-2640",
    "zipper_mouth": "1f910",
    "flag_mv": "1f1f2-1f1fb",
    "person_climbing_tone5": "1f9d7-1f3ff",
    "man_white_haired_tone2": "1f468-1f3fc-1f9b3",
    "peace": "262e",
    "man_in_steamy_room_tone3": "1f9d6-1f3fd-2642",
    "woman_raising_hand_tone5": "1f64b-1f3ff-2640",
    "adult_tone2": "1f9d1-1f3fc",
    "woman_rowing_boat_tone1": "1f6a3-1f3fb-2640",
    "man_tipping_hand_tone3": "1f481-1f3fd-2642",
    "man_bowing_tone3": "1f647-1f3fd-2642",
    "man_firefighter_tone5": "1f468-1f3ff-1f692",
    "person_in_steamy_room_tone5": "1f9d6-1f3ff",
    "person_shrugging": "1f937",
    "blond-haired_man_tone4": "1f471-1f3fe-2642",
    "man_lifting_weights_tone1": "1f3cb-1f3fb-2642",
    "woman_mountain_biking_tone2": "1f6b5-1f3fc-2640",
    "man_detective_tone4": "1f575-1f3fe-2642",
    "point_up_tone2": "261d-1f3fc",
    "man_artist_tone1": "1f468-1f3fb-1f3a8",
    "woman_golfing_tone1": "1f3cc-1f3fb-2640",
    "person_gesturing_no_tone4": "1f645-1f3fe",
    "snowboarder_tone3": "1f3c2-1f3fd",
    "flag_br": "1f1e7-1f1f7",
    "blond-haired_woman_tone1": "1f471-1f3fb-2640",
    "person_rowing_boat_tone4": "1f6a3-1f3fe",
    "clown": "1f921",
    "man_superhero_tone2": "1f9b8-1f3fc-2642",
    "flag_ve": "1f1fb-1f1ea",
    "flag_bd": "1f1e7-1f1e9",
    "couple_ww": "1f469-2764-1f469",
    "regional_indicator_n": "1f1f3",
    "man_lifting_weights_tone2": "1f3cb-1f3fc-2642",
    "money_mouth": "1f911",
    "merman_tone1": "1f9dc-1f3fb-2642",
    "dancer_tone4": "1f483-1f3fe",
    "breast_feeding_tone1": "1f931-1f3fb",
    "man_getting_haircut_tone1": "1f487-1f3fb-2642",
    "flag_tf": "1f1f9-1f1eb",
    "mrs_claus_tone3": "1f936-1f3fd",
    "man_dancing_tone4": "1f57a-1f3fe",
    "woman_student": "1f469-1f393",
    "metal": "1f918",
    "person_rowing_boat_tone5": "1f6a3-1f3ff",
    "woman_gesturing_ok_tone5": "1f646-1f3ff-2640",
    "levitate_tone3": "1f574-1f3fd",
    "man_tone2": "1f468-1f3fc",
    "woman_detective_tone3": "1f575-1f3fd-2640",
    "flag_la": "1f1f1-1f1e6",
    "foot_tone4": "1f9b6-1f3fe",
    "flag_ly": "1f1f1-1f1fe",
    "man_gesturing_no_tone4": "1f645-1f3fe-2642",
    "woman_playing_water_polo_tone5": "1f93d-1f3ff-2640",
    "flag_ws": "1f1fc-1f1f8",
    "person_gesturing_no": "1f645",
    "flag_mt": "1f1f2-1f1f9",
    "flag_ci": "1f1e8-1f1ee",
    "man_playing_water_polo": "1f93d-2642",
    "flag_mk": "1f1f2-1f1f0",
    "selfie_tone2": "1f933-1f3fc",
    "woman_pilot_tone5": "1f469-1f3ff-2708",
    "pray_tone5": "1f64f-1f3ff",
    "ear_tone2": "1f442-1f3fc",
    "track_previous": "23ee",
    "woman_in_lotus_position_tone1": "1f9d8-1f3fb-2640",
    "woman_with_headscarf_tone5": "1f9d5-1f3ff",
    "woman_fairy_tone5": "1f9da-1f3ff-2640",
    "regional_indicator_w": "1f1fc",
    "leg_tone5": "1f9b5-1f3ff",
    "regional_indicator_z": "1f1ff",
    "man_dancing_tone1": "1f57a-1f3fb",
    "man_wearing_turban_tone4": "1f473-1f3fe-2642",
    "flag_pg": "1f1f5-1f1ec",
    "cloud_rain": "1f327",
    "selfie_tone5": "1f933-1f3ff",
    "flag_gy": "1f1ec-1f1fe",
    "race_car": "1f3ce",
    "angel_tone3": "1f47c-1f3fd",
    "flag_fi": "1f1eb-1f1ee",
    "leg_tone4": "1f9b5-1f3fe",
    "woman_in_lotus_position_tone2": "1f9d8-1f3fc-2640",
    "woman_astronaut_tone5": "1f469-1f3ff-1f680",
    "mage_tone4": "1f9d9-1f3fe",
    "timer": "23f2",
    "bride_with_veil_tone3": "1f470-1f3fd",
    "skull_crossbones": "2620",
    "woman_firefighter_tone4": "1f469-1f3fe-1f692",
    "blond-haired_man_tone5": "1f471-1f3ff-2642",
    "merman_tone5": "1f9dc-1f3ff-2642",
    "flag_bw": "1f1e7-1f1fc",
    "man_frowning_tone2": "1f64d-1f3fc-2642",
    "fingers_crossed_tone2": "1f91e-1f3fc",
    "woman_white_haired_tone3": "1f469-1f3fd-1f9b3",
    "flag_om": "1f1f4-1f1f2",
    "man_frowning": "1f64d-2642",
    "call_me_tone2": "1f919-1f3fc",
    "flag_ie": "1f1ee-1f1ea",
    "wave_tone4": "1f44b-1f3fe",
    "flag_bn": "1f1e7-1f1f3",
    "flag_am": "1f1e6-1f1f2",
    "person_getting_massage_tone4": "1f486-1f3fe",
    "thinking": "1f914",
    "regional_indicator_v": "1f1fb",
    "man_factory_worker_tone3": "1f468-1f3fd-1f3ed",
    "man_juggling_tone3": "1f939-1f3fd-2642",
    "man_detective_tone2": "1f575-1f3fc-2642",
    "woman_walking_tone5": "1f6b6-1f3ff-2640",
    "family_woman_boy_boy": "1f469-1f466-1f466",
    "flag_zm": "1f1ff-1f1f2",
    "vampire_tone2": "1f9db-1f3fc",
    "point_left_tone3": "1f448-1f3fd",
    "woman_vampire_tone2": "1f9db-1f3fc-2640",
    "bearded_person_tone1": "1f9d4-1f3fb",
    "person_swimming": "1f3ca",
    "woman_technologist": "1f469-1f4bb",
    "man_artist": "1f468-1f3a8",
    "person_running_tone2": "1f3c3-1f3fc",
    "call_me_tone1": "1f919-1f3fb",
    "person_frowning_tone4": "1f64d-1f3fe",
    "flag_lr": "1f1f1-1f1f7",
    "point_right_tone4": "1f449-1f3fe",
    "man_scientist_tone5": "1f468-1f3ff-1f52c",
    "man_bald": "1f468-1f9b2",
    "person_pouting": "1f64e",
    "man_juggling_tone4": "1f939-1f3fe-2642",
    "flag_fo": "1f1eb-1f1f4",
    "raised_back_of_hand_tone1": "1f91a-1f3fb",
    "fist_tone2": "270a-1f3fc",
    "person_playing_handball_tone5": "1f93e-1f3ff",
    "man_raising_hand_tone4": "1f64b-1f3fe-2642",
    "left_facing_fist_tone1": "1f91b-1f3fb",
    "woman_wearing_turban_tone5": "1f473-1f3ff-2640",
    "woman_gesturing_no_tone1": "1f645-1f3fb-2640",
    "flag_ph": "1f1f5-1f1ed",
    "woman_fairy_tone2": "1f9da-1f3fc-2640",
    "flag_bb": "1f1e7-1f1e7",
    "man_in_tuxedo_tone3": "1f935-1f3fd",
    "woman_pouting_tone2": "1f64e-1f3fc-2640",
    "man_singer": "1f468-1f3a4",
    "person_bouncing_ball_tone2": "26f9-1f3fc",
    "child_tone2": "1f9d2-1f3fc",
    "man_tone3": "1f468-1f3fd",
    "child_tone1": "1f9d2-1f3fb",
    "person_bouncing_ball_tone4": "26f9-1f3fe",
    "person_bouncing_ball_tone3": "26f9-1f3fd",
    "digit_six": "0036",
    "flag_in": "1f1ee-1f1f3",
    "person_wearing_turban": "1f473",
    "mountain_snow": "1f3d4",
    "princess_tone1": "1f478-1f3fb",
    "construction_worker_tone1": "1f477-1f3fb",
    "flag_gp": "1f1ec-1f1f5",
    "person_shrugging_tone3": "1f937-1f3fd",
    "woman_playing_water_polo": "1f93d-2640",
    "man_in_lotus_position_tone5": "1f9d8-1f3ff-2642",
    "man_zombie": "1f9df-2642",
    "flag_gn": "1f1ec-1f1f3",
    "man_pouting_tone2": "1f64e-1f3fc-2642",
    "tone5": "1f3ff",
    "person_running": "1f3c3",
    "point_up_2_tone4": "1f446-1f3fe",
    "woman_superhero": "1f9b8-2640",
    "french_bread": "1f956",
    "flag_ps": "1f1f5-1f1f8",
    "woman_vampire_tone4": "1f9db-1f3fe-2640",
    "man_astronaut_tone1": "1f468-1f3fb-1f680",
    "middle_finger_tone2": "1f595-1f3fc",
    "woman_bouncing_ball_tone4": "26f9-1f3fe-2640",
    "man_office_worker_tone3": "1f468-1f3fd-1f4bc",
    "man_bowing_tone4": "1f647-1f3fe-2642",
    "regional_indicator_m": "1f1f2",
    "woman_in_steamy_room_tone2": "1f9d6-1f3fc-2640",
    "man_getting_haircut_tone5": "1f487-1f3ff-2642",
    "flag_ax": "1f1e6-1f1fd",
    "clap_tone2": "1f44f-1f3fc",
    "anger_right": "1f5ef",
    "woman_tone4": "1f469-1f3fe",
    "family_mwg": "1f468-1f469-1f467",
    "raised_hands_tone2": "1f64c-1f3fc",
    "man_mechanic_tone4": "1f468-1f3fe-1f527",
    "flag_kh": "1f1f0-1f1ed",
    "woman_bowing_tone1": "1f647-1f3fb-2640",
    "girl_tone5": "1f467-1f3ff",
    "man_superhero": "1f9b8-2642",
    "person_doing_cartwheel_tone3": "1f938-1f3fd",
    "motorboat": "1f6e5",
    "flag_xk": "1f1fd-1f1f0",
    "curly_haired": "1f9b1",
    "mrs_claus_tone2": "1f936-1f3fc",
    "woman_health_worker_tone4": "1f469-1f3fe-2695",
    "woman_curly_haired_tone4": "1f469-1f3fe-1f9b1",
    "digit_nine": "0039",
    "woman_farmer_tone2": "1f469-1f3fc-1f33e",
    "woman_raising_hand_tone4": "1f64b-1f3fe-2640",
    "man_in_lotus_position_tone3": "1f9d8-1f3fd-2642",
    "left_facing_fist_tone5": "1f91b-1f3ff",
    "fairy_tone1": "1f9da-1f3fb",
    "woman_facepalming_tone2": "1f926-1f3fc-2640",
    "flag_hr": "1f1ed-1f1f7",
    "person_facepalming_tone4": "1f926-1f3fe",
    "flag_ky": "1f1f0-1f1fe",
    "flag_li": "1f1f1-1f1ee",
    "person_swimming_tone3": "1f3ca-1f3fd",
    "fork_knife_plate": "1f37d",
    "flag_gd": "1f1ec-1f1e9",
    "thumbsdown_tone3": "1f44e-1f3fd",
    "wave_tone1": "1f44b-1f3fb",
    "police_officer_tone4": "1f46e-1f3fe",
    "second_place": "1f948",
    "man_cartwheeling_tone4": "1f938-1f3fe-2642",
    "woman_judge_tone5": "1f469-1f3ff-2696",
    "guard_tone4": "1f482-1f3fe",
    "point_right_tone5": "1f449-1f3ff",
    "raised_hand_tone1": "270b-1f3fb",
    "island": "1f3dd",
    "flag_ch": "1f1e8-1f1ed",
    "man_bouncing_ball_tone4": "26f9-1f3fe-2642",
    "girl_tone1": "1f467-1f3fb",
    "santa_tone3": "1f385-1f3fd",
    "muscle_tone2": "1f4aa-1f3fc",
    "kiss_ww": "1f469-2764-1f48b-1f469",
    "flag_sm": "1f1f8-1f1f2",
    "woman_shrugging_tone1": "1f937-1f3fb-2640",
    "flag_bz": "1f1e7-1f1ff",
    "flag_se": "1f1f8-1f1ea",
    "cricket_game": "1f3cf",
    "family_man_girl_girl": "1f468-1f467-1f467",
    "flag_uz": "1f1fa-1f1ff",
    "point_up_tone3": "261d-1f3fd",
    "man_astronaut_tone5": "1f468-1f3ff-1f680",
    "oil": "1f6e2",
    "man_supervillain_tone3": "1f9b9-1f3fd-2642",
    "man_singer_tone5": "1f468-1f3ff-1f3a4",
    "person_tipping_hand_tone1": "1f481-1f3fb",
    "flag_ye": "1f1fe-1f1ea",
    "woman_red_haired_tone3": "1f469-1f3fd-1f9b0",
    "woman_playing_water_polo_tone4": "1f93d-1f3fe-2640",
    "man_fairy": "1f9da-2642",
    "man_shrugging_tone3": "1f937-1f3fd-2642",
    "regional_indicator_p": "1f1f5",
    "radioactive": "2622",
    "person_tipping_hand": "1f481",
    "man_in_steamy_room_tone2": "1f9d6-1f3fc-2642",
    "ping_pong": "1f3d3",
    "flag_na": "1f1f3-1f1e6",
    "man_tone1": "1f468-1f3fb",
    "pregnant_woman_tone1": "1f930-1f3fb",
    "flag_pa": "1f1f5-1f1e6",
    "ear_tone4": "1f442-1f3fe",
    "flag_bq": "1f1e7-1f1f6",
    "regional_indicator_f": "1f1eb",
    "flag_bv": "1f1e7-1f1fb",
    "man_curly_haired_tone1": "1f468-1f3fb-1f9b1",
    "woman_superhero_tone2": "1f9b8-1f3fc-2640",
    "man_climbing_tone3": "1f9d7-1f3fd-2642",
    "prince_tone2": "1f934-1f3fc",
    "man_farmer_tone5": "1f468-1f3ff-1f33e",
    "superhero_tone4": "1f9b8-1f3fe",
    "love_you_gesture_tone4": "1f91f-1f3fe",
    "man_surfing_tone4": "1f3c4-1f3fe-2642",
    "woman_tone1": "1f469-1f3fb",
    "woman_wearing_turban": "1f473-2640",
    "clap_tone5": "1f44f-1f3ff",
    "digit_seven": "0037",
    "person_rowing_boat_tone1": "1f6a3-1f3fb",
    "woman_scientist_tone1": "1f469-1f3fb-1f52c",
    "woman_superhero_tone5": "1f9b8-1f3ff-2640",
    "hugging": "1f917",
    "regional_indicator_l": "1f1f1",
    "man_police_officer_tone4": "1f46e-1f3fe-2642",
    "police_officer_tone1": "1f46e-1f3fb",
    "man_gesturing_ok": "1f646-2642",
    "woman_in_steamy_room_tone4": "1f9d6-1f3fe-2640",
    "cheese": "1f9c0",
    "man_tipping_hand_tone1": "1f481-1f3fb-2642",
    "woman_frowning_tone5": "1f64d-1f3ff-2640",
    "man_scientist": "1f468-1f52c",
    "adult_tone4": "1f9d1-1f3fe",
    "older_woman_tone1": "1f475-1f3fb",
    "flag_tn": "1f1f9-1f1f3",
    "woman_rowing_boat_tone3": "1f6a3-1f3fd-2640",
    "woman_factory_worker_tone3": "1f469-1f3fd-1f3ed",
    "man_police_officer_tone5": "1f46e-1f3ff-2642",
    "woman_lifting_weights_tone2": "1f3cb-1f3fc-2640",
    "point_left_tone1": "1f448-1f3fb",
    "flag_wf": "1f1fc-1f1eb",
    "man_firefighter_tone3": "1f468-1f3fd-1f692",
    "merman_tone3": "1f9dc-1f3fd-2642",
    "slight_frown": "1f641",
    "woman_singer_tone4": "1f469-1f3fe-1f3a4",
    "man_teacher_tone4": "1f468-1f3fe-1f3eb",
    "man_health_worker_tone3": "1f468-1f3fd-2695",
    "man_tipping_hand_tone2": "1f481-1f3fc-2642",
    "rolling_eyes": "1f644",
    "punch_tone4": "1f44a-1f3fe",
    "flag_tc": "1f1f9-1f1e8",
    "person_gesturing_no_tone2": "1f645-1f3fc",
    "man_technologist_tone1": "1f468-1f3fb-1f4bb",
    "v_tone3": "270c-1f3fd",
    "man_pilot_tone3": "1f468-1f3fd-2708",
    "man_with_chinese_cap_tone1": "1f472-1f3fb",
    "person_golfing_tone3": "1f3cc-1f3fd",
    "woman_tone5": "1f469-1f3ff",
    "snowboarder_tone1": "1f3c2-1f3fb",
    "santa_tone1": "1f385-1f3fb",
    "family_man_girl": "1f468-1f467",
    "tools": "1f6e0",
    "man_climbing_tone5": "1f9d7-1f3ff-2642",
    "man_biking_tone2": "1f6b4-1f3fc-2642",
    "flag_ki": "1f1f0-1f1ee",
    "flag_eu": "1f1ea-1f1fa",
    "man_facepalming_tone3": "1f926-1f3fd-2642",
    "man_student_tone5": "1f468-1f3ff-1f393",
    "person_in_lotus_position_tone2": "1f9d8-1f3fc",
    "ballot_box": "1f5f3",
    "man_facepalming_tone1": "1f926-1f3fb-2642",
    "woman_red_haired": "1f469-1f9b0",
    "woman_pouting_tone4": "1f64e-1f3fe-2640",
    "flag_km": "1f1f0-1f1f2",
    "woman_juggling_tone2": "1f939-1f3fc-2640",
    "man_white_haired": "1f468-1f9b3",
    "beach": "1f3d6",
    "wilted_rose": "1f940",
    "person_walking_tone2": "1f6b6-1f3fc",
    "flag_sv": "1f1f8-1f1fb",
    "man_golfing_tone5": "1f3cc-1f3ff-2642",
    "woman_getting_face_massage_tone2": "1f486-1f3fc-2640",
    "woman_golfing_tone3": "1f3cc-1f3fd-2640",
    "woman_student_tone1": "1f469-1f3fb-1f393",
    "adult_tone5": "1f9d1-1f3ff",
    "person_walking_tone1": "1f6b6-1f3fb",
    "woman_in_steamy_room_tone3": "1f9d6-1f3fd-2640",
    "nose_tone2": "1f443-1f3fc",
    "pregnant_woman_tone5": "1f930-1f3ff",
    "person_getting_massage_tone3": "1f486-1f3fd",
    "family_mmgb": "1f468-1f468-1f467-1f466",
    "man_bald_tone1": "1f468-1f3fb-1f9b2",
    "person_getting_massage_tone1": "1f486-1f3fb",
    "woman_mountain_biking_tone5": "1f6b5-1f3ff-2640",
    "middle_finger_tone5": "1f595-1f3ff",
    "flag_gl": "1f1ec-1f1f1",
    "man_elf_tone5": "1f9dd-1f3ff-2642",
    "woman_golfing_tone2": "1f3cc-1f3fc-2640",
    "woman_curly_haired_tone1": "1f469-1f3fb-1f9b1",
    "woman_vampire_tone1": "1f9db-1f3fb-2640",
    "flag_kr": "1f1f0-1f1f7",
    "flag_cc": "1f1e8-1f1e8",
    "man_artist_tone3": "1f468-1f3fd-1f3a8",
    "cruise_ship": "1f6f3",
    "fingers_crossed_tone1": "1f91e-1f3fb",
    "flag_dg": "1f1e9-1f1ec",
    "star_struck": "1f929",
    "right_facing_fist_tone2": "1f91c-1f3fc",
    "point_up_2_tone5": "1f446-1f3ff",
    "woman_office_worker_tone4": "1f469-1f3fe-1f4bc",
    "woman_detective_tone4": "1f575-1f3fe-2640",
    "flag_cp": "1f1e8-1f1f5",
    "blond-haired_man_tone1": "1f471-1f3fb-2642",
    "family_mwbb": "1f468-1f469-1f466-1f466",
    "bride_with_veil_tone2": "1f470-1f3fc",
    "woman_mechanic_tone4": "1f469-1f3fe-1f527",
    "person_walking": "1f6b6",
    "person_mountain_biking": "1f6b5",
    "card_box": "1f5c3",
    "flag_lu": "1f1f1-1f1fa",
    "person_in_steamy_room_tone4": "1f9d6-1f3fe",
    "man_running": "1f3c3-2642",
    "man_elf_tone3": "1f9dd-1f3fd-2642",
    "muscle_tone1": "1f4aa-1f3fb",
    "man_lifting_weights": "1f3cb-2642",
    "detective_tone2": "1f575-1f3fc",
    "woman_gesturing_ok": "1f646-2640",
    "man_construction_worker_tone5": "1f477-1f3ff-2642",
    "person_surfing_tone5": "1f3c4-1f3ff",
    "flag_ga": "1f1ec-1f1e6",
    "flag_pf": "1f1f5-1f1eb",
    "woman_health_worker_tone2": "1f469-1f3fc-2695",
    "selfie_tone1": "1f933-1f3fb",
    "point_up_tone5": "261d-1f3ff",
    "vulcan_tone2": "1f596-1f3fc",
    "asterisk_symbol": "002a",
    "foot_tone5": "1f9b6-1f3ff",
    "man_golfing_tone3": "1f3cc-1f3fd-2642",
    "woman_scientist_tone2": "1f469-1f3fc-1f52c",
    "person_bowing_tone3": "1f647-1f3fd",
    "man_in_lotus_position_tone1": "1f9d8-1f3fb-2642",
    "woman_cartwheeling_tone3": "1f938-1f3fd-2640",
    "flag_si": "1f1f8-1f1ee",
    "nail_care_tone4": "1f485-1f3fe",
    "person_bowing_tone1": "1f647-1f3fb",
    "man_swimming_tone4": "1f3ca-1f3fe-2642",
    "military_medal": "1f396",
    "person_wearing_turban_tone2": "1f473-1f3fc",
    "woman_tipping_hand_tone2": "1f481-1f3fc-2640",
    "woman_scientist_tone5": "1f469-1f3ff-1f52c",
    "v_tone4": "270c-1f3fe",
    "man_construction_worker_tone4": "1f477-1f3fe-2642",
    "woman_raising_hand": "1f64b-2640",
    "man_construction_worker_tone2": "1f477-1f3fc-2642",
    "flag_vu": "1f1fb-1f1fa",
    "horse_racing_tone2": "1f3c7-1f3fc",
    "flag_ag": "1f1e6-1f1ec",
    "man_bouncing_ball_tone1": "26f9-1f3fb-2642",
    "woman_superhero_tone3": "1f9b8-1f3fd-2640",
    "man_mountain_biking_tone5": "1f6b5-1f3ff-2642",
    "fingers_crossed": "1f91e",
    "man_red_haired_tone5": "1f468-1f3ff-1f9b0",
    "regional_indicator_t": "1f1f9",
    "flag_sr": "1f1f8-1f1f7",
    "person_rowing_boat_tone3": "1f6a3-1f3fd",
    "person_raising_hand_tone3": "1f64b-1f3fd",
    "man_walking": "1f6b6-2642",
    "man_shrugging_tone4": "1f937-1f3fe-2642",
    "woman_gesturing_ok_tone2": "1f646-1f3fc-2640",
    "flag_bi": "1f1e7-1f1ee",
    "flag_cn": "1f1e8-1f1f3",
    "flag_nu": "1f1f3-1f1fa",
    "family_wwb": "1f469-1f469-1f466",
    "flag_tg": "1f1f9-1f1ec",
    "woman_guard": "1f482-2640",
    "man_getting_face_massage_tone2": "1f486-1f3fc-2642",
    "v_tone1": "270c-1f3fb",
    "woman_playing_handball_tone2": "1f93e-1f3fc-2640",
    "man_mountain_biking_tone2": "1f6b5-1f3fc-2642",
    "clap_tone3": "1f44f-1f3fd",
    "woman_gesturing_no_tone5": "1f645-1f3ff-2640",
    "woman_playing_handball_tone1": "1f93e-1f3fb-2640",
    "flag_qa": "1f1f6-1f1e6",
    "family_wwgb": "1f469-1f469-1f467-1f466",
    "person_in_bed_tone5": "1f6cc-1f3ff",
    "man_pouting": "1f64e-2642",
    "woman_mage_tone5": "1f9d9-1f3ff-2640",
    "flag_mr": "1f1f2-1f1f7",
    "person_tipping_hand_tone4": "1f481-1f3fe",
    "person_gesturing_ok_tone5": "1f646-1f3ff",
    "open_hands_tone5": "1f450-1f3ff",
    "man_farmer_tone1": "1f468-1f3fb-1f33e",
    "man_with_chinese_cap": "1f472",
    "person_mountain_biking_tone1": "1f6b5-1f3fb",
    "regional_indicator_a": "1f1e6",
    "man_guard_tone3": "1f482-1f3fd-2642",
    "woman_construction_worker_tone4": "1f477-1f3fe-2640",
    "man_health_worker": "1f468-2695",
    "flag_eg": "1f1ea-1f1ec",
    "flag_gs": "1f1ec-1f1f8",
    "man_supervillain_tone4": "1f9b9-1f3fe-2642",
    "supervillain_tone1": "1f9b9-1f3fb",
    "motorcycle": "1f3cd",
    "man_pouting_tone1": "1f64e-1f3fb-2642",
    "ok_hand_tone5": "1f44c-1f3ff",
    "woman_with_headscarf_tone4": "1f9d5-1f3fe",
    "person_lifting_weights_tone1": "1f3cb-1f3fb",
    "woman_firefighter_tone2": "1f469-1f3fc-1f692",
    "woman_technologist_tone2": "1f469-1f3fc-1f4bb",
    "flag_it": "1f1ee-1f1f9",
    "open_hands_tone1": "1f450-1f3fb",
    "flag_sd": "1f1f8-1f1e9",
    "woman_pilot": "1f469-2708",
    "older_adult_tone4": "1f9d3-1f3fe",
    "flag_af": "1f1e6-1f1eb",
    "woman_construction_worker_tone2": "1f477-1f3fc-2640",
    "woman_mountain_biking_tone3": "1f6b5-1f3fd-2640",
    "man_shrugging_tone5": "1f937-1f3ff-2642",
    "man_factory_worker_tone4": "1f468-1f3fe-1f3ed",
    "woman_getting_face_massage_tone3": "1f486-1f3fd-2640",
    "flag_bt": "1f1e7-1f1f9",
    "man_bald_tone5": "1f468-1f3ff-1f9b2",
    "woman_cartwheeling_tone4": "1f938-1f3fe-2640",
    "man_in_tuxedo_tone1": "1f935-1f3fb",
    "flag_cy": "1f1e8-1f1fe",
    "woman_running_tone1": "1f3c3-1f3fb-2640",
    "nail_care_tone2": "1f485-1f3fc",
    "woman_facepalming": "1f926-2640",
    "asterisk": "002a-20e3",
    "person_playing_water_polo_tone3": "1f93d-1f3fd",
    "flag_dk": "1f1e9-1f1f0",
    "flag_nl": "1f1f3-1f1f1",
    "man_detective": "1f575-2642",
    "woman_judge_tone3": "1f469-1f3fd-2696",
    "call_me_tone3": "1f919-1f3fd",
    "woman_getting_haircut_tone1": "1f487-1f3fb-2640",
    "woman_juggling_tone4": "1f939-1f3fe-2640",
    "man_mountain_biking_tone1": "1f6b5-1f3fb-2642",
    "woman_mountain_biking": "1f6b5-2640",
    "person_juggling_tone4": "1f939-1f3fe",
    "field_hockey": "1f3d1"
  };
}
